@@ -22,14 +22,11 @@ if [[ "${TEST_SPHINX}" == "true" ]]; then
     cd doc
     make html
     make man
-    make latex
-    cd _build/latex
-    export LATEXMKOPTS="-halt-on-error -xelatex -silent"
-    make all || {
+    make latexpdf LATEXMKOPTS="-halt-on-error -xelatex -silent" || {
         echo "An error had occured during the LaTeX build";
-        tail -n 1000 *.log;
+        tail -n 1000 _build/latex/*.log;
         sleep 1; # A guard against travis running tail concurrently.
-        exit -1;
+        exit 1;
     }
 fi
 
