@@ -1,6 +1,14 @@
-from sympy import symbols, re, im, sign, I, Abs, Symbol, \
-     cos, sin, sqrt, conjugate, log, acos, asin, E, pi, \
-     Matrix, diff, integrate, trigsimp, S, Rational
+from sympy.core.function import diff
+from sympy.core.numbers import (E, I, Rational, pi)
+from sympy.core.singleton import S
+from sympy.core.symbol import (Symbol, symbols)
+from sympy.functions.elementary.complexes import (Abs, conjugate, im, re, sign)
+from sympy.functions.elementary.exponential import log
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.trigonometric import (acos, asin, cos, sin, atan2, atan)
+from sympy.integrals.integrals import integrate
+from sympy.matrices.dense import Matrix
+from sympy.simplify.trigsimp import trigsimp
 from sympy.algebras.quaternion import Quaternion
 from sympy.testing.pytest import raises
 
@@ -133,6 +141,60 @@ def test_quaternion_functions():
     n = Symbol('n', integer=True)
     raises(TypeError, lambda: q1**n)
 
+    assert Quaternion(22, 23, 55, 8).scalar_part() == 22
+    assert Quaternion(w, x, y, z).scalar_part() == w
+
+    assert Quaternion(22, 23, 55, 8).vector_part() == Quaternion(0, 23, 55, 8)
+    assert Quaternion(w, x, y, z).vector_part() == Quaternion(0, x, y, z)
+
+    assert q1.axis() == Quaternion(0, 2*sqrt(29)/29, 3*sqrt(29)/29, 4*sqrt(29)/29)
+    assert q1.axis().pow(2) == Quaternion(-1, 0, 0, 0)
+    assert q0.axis().scalar_part() == 0
+    assert q.axis() == Quaternion(0, x/sqrt(x**2 + y**2 + z**2), y/sqrt(x**2 + y**2 + z**2), z/sqrt(x**2 + y**2 + z**2))
+
+    assert q0.is_pure() == True
+    assert q1.is_pure() == False
+    assert Quaternion(0, 0, 0, 3).is_pure() == True
+    assert Quaternion(0, 2, 10, 3).is_pure() == True
+    assert Quaternion(w, 2, 10, 3).is_pure() == None
+
+    assert q1.angle() == atan(sqrt(29))
+    assert q.angle() == atan2(sqrt(x**2 + y**2 + z**2), w)
+
+    assert Quaternion.arc_coplanar(q1, Quaternion(2, 4, 6, 8)) == True
+    assert Quaternion.arc_coplanar(q1, Quaternion(1, -2, -3, -4)) == True
+    assert Quaternion.arc_coplanar(q1, Quaternion(1, 8, 12, 16)) == True
+    assert Quaternion.arc_coplanar(q1, Quaternion(1, 2, 3, 4)) == True
+    assert Quaternion.arc_coplanar(q1, Quaternion(w, 4, 6, 8)) == True
+    assert Quaternion.arc_coplanar(q1, Quaternion(2, 7, 4, 1)) == False
+    assert Quaternion.arc_coplanar(q1, Quaternion(w, x, y, z)) == None
+    raises(ValueError, lambda: Quaternion.arc_coplanar(q1, q0))
+
+    assert Quaternion.vector_coplanar(Quaternion(0, 8, 12, 16), Quaternion(0, 4, 6, 8), Quaternion(0, 2, 3, 4)) == True
+    assert Quaternion.vector_coplanar(Quaternion(0, 0, 0, 0), Quaternion(0, 4, 6, 8), Quaternion(0, 2, 3, 4)) == True
+    assert Quaternion.vector_coplanar(Quaternion(0, 8, 2, 6), Quaternion(0, 1, 6, 6), Quaternion(0, 0, 3, 4)) == False
+    assert Quaternion.vector_coplanar(Quaternion(0, 1, 3, 4), Quaternion(0, 4, w, 6), Quaternion(0, 6, 8, 1)) == None
+    raises(ValueError, lambda: Quaternion.vector_coplanar(q0, Quaternion(0, 4, 6, 8), q1))
+
+    assert Quaternion(0, 1, 2, 3).parallel(Quaternion(0, 2, 4, 6)) == True
+    assert Quaternion(0, 1, 2, 3).parallel(Quaternion(0, 2, 2, 6)) == False
+    assert Quaternion(0, 1, 2, 3).parallel(Quaternion(w, x, y, 6)) == None
+    raises(ValueError, lambda: q0.parallel(q1))
+
+    assert Quaternion(0, 1, 2, 3).orthogonal(Quaternion(0, -2, 1, 0)) == True
+    assert Quaternion(0, 2, 4, 7).orthogonal(Quaternion(0, 2, 2, 6)) == False
+    assert Quaternion(0, 2, 4, 7).orthogonal(Quaternion(w, x, y, 6)) == None
+    raises(ValueError, lambda: q0.orthogonal(q1))
+
+    assert q1.index_vector() == Quaternion(0, 2*sqrt(870)/29, 3*sqrt(870)/29, 4*sqrt(870)/29)
+    assert Quaternion(0, 3, 9, 4).index_vector() == Quaternion(0, 3, 9, 4)
+
+    assert Quaternion(4, 3, 9, 4).mensor() == log(sqrt(122))
+    assert Quaternion(3, 3, 0, 2).mensor() == log(sqrt(22))
+
+    assert q0.is_zero_quaternion() == True
+    assert q1.is_zero_quaternion() == False
+    assert Quaternion(w, 0, 0, 0).is_zero_quaternion() == None
 
 def test_quaternion_conversions():
     q1 = Quaternion(1, 2, 3, 4)
@@ -190,8 +252,8 @@ def test_quaternion_multiplication():
     q2 = Quaternion(1, 2, 3, 5)
     q3 = Quaternion(1, 1, 1, y)
 
-    assert Quaternion._generic_mul(4, 1) == 4
-    assert Quaternion._generic_mul(4, q1) == Quaternion(12 + 16*I, 8 + 20*I, 0, 28 + 32*I)
+    assert Quaternion._generic_mul(S(4), S.One) == 4
+    assert Quaternion._generic_mul(S(4), q1) == Quaternion(12 + 16*I, 8 + 20*I, 0, 28 + 32*I)
     assert q2.mul(2) == Quaternion(2, 4, 6, 10)
     assert q2.mul(q3) == Quaternion(-5*y - 4, 3*y - 2, 9 - 2*y, y + 4)
     assert q2.mul(q3) == q2*q3
