@@ -1,4 +1,4 @@
-from typing import Any, Set
+from typing import Any, Set as tSet
 
 from functools import reduce
 from itertools import permutations
@@ -15,10 +15,12 @@ from sympy.core.symbol import Str
 from sympy.core.sympify import _sympify
 from sympy.functions import factorial
 from sympy.matrices import ImmutableDenseMatrix as Matrix
-from sympy.simplify import simplify
 from sympy.solvers import solve
 
-from sympy.utilities.exceptions import SymPyDeprecationWarning
+from sympy.utilities.exceptions import (sympy_deprecation_warning,
+                                        SymPyDeprecationWarning,
+                                        ignore_warnings)
+
 
 # TODO you are a bit excessive in the use of Dummies
 # TODO dummy point, literal field
@@ -71,12 +73,11 @@ class Manifold(Basic):
         obj = super().__new__(cls, name, dim)
 
         obj.patches = _deprecated_list(
-            "Manifold.patches",
-            "external container for registry",
-            19321,
-            "1.7",
-            []
-        )
+            """
+            Manifold.patches is deprecated. The Manifold object is now
+            immutable. Instead use a separate list to keep track of the
+            patches.
+            """, [])
         return obj
 
     @property
@@ -138,12 +139,11 @@ class Patch(Basic):
 
         obj.manifold.patches.append(obj) # deprecated
         obj.coord_systems = _deprecated_list(
-            "Patch.coord_systems",
-            "external container for registry",
-            19321,
-            "1.7",
-            []
-        )
+            """
+            Patch.coord_systms is deprecated. The Patch class is now
+            immutable. Instead use a separate list to keep track of coordinate
+            systems.
+            """, [])
         return obj
 
     @property
@@ -282,12 +282,20 @@ class CoordSystem(Basic):
                       for i in range(patch.dim)]
                 )
             else:
-                SymPyDeprecationWarning(
-                    feature="Class signature 'names' of CoordSystem",
-                    useinstead="class signature 'symbols'",
-                    issue=19321,
-                    deprecated_since_version="1.7"
-                ).warn()
+                sympy_deprecation_warning(
+                    f"""
+The 'names' argument to CoordSystem is deprecated. Use 'symbols' instead. That
+is, replace
+
+    CoordSystem(..., names={names})
+
+with
+
+    CoordSystem(..., symbols=[{', '.join(["Symbol(" + repr(n) + ", real=True)" for n in names])}])
+                    """,
+                    deprecated_since_version="1.7",
+                    active_deprecations_target="deprecated-diffgeom-mutable",
+                )
                 symbols = Tuple(
                     *[Symbol(n, real=True) for n in names]
                 )
@@ -297,12 +305,18 @@ class CoordSystem(Basic):
                 if isinstance(s, Symbol):
                     syms.append(Symbol(s.name, **s._assumptions.generator))
                 elif isinstance(s, str):
-                    SymPyDeprecationWarning(
-                        feature="Passing str as coordinate symbol's name",
-                        useinstead="Symbol which contains the name and assumption for coordinate symbol",
-                        issue=19321,
-                        deprecated_since_version="1.7"
-                    ).warn()
+                    sympy_deprecation_warning(
+                        f"""
+
+Passing a string as the coordinate symbol name to CoordSystem is deprecated.
+Pass a Symbol with the appropriate name and assumptions instead.
+
+That is, replace {s} with Symbol({s!r}, real=True).
+                        """,
+
+                        deprecated_since_version="1.7",
+                        active_deprecations_target="deprecated-diffgeom-mutable",
+                    )
                     syms.append(Symbol(s, real=True))
             symbols = Tuple(*syms)
 
@@ -329,12 +343,11 @@ class CoordSystem(Basic):
 
         # Add deprecated attributes
         obj.transforms = _deprecated_dict(
-            "Mutable CoordSystem.transforms",
-            "'relations' parameter in class signature",
-            19321,
-            "1.7",
-            {}
-        )
+            """
+            CoordSystem.transforms is deprecated. The CoordSystem class is now
+            immutable. Use the 'relations' keyword argument to the
+            CoordSystems() constructor to specify relations.
+            """, {})
         obj._names = [str(n) for n in symbols]
         obj.patch.coord_systems.append(obj) # deprecated
         obj._dummies = [Dummy(str(n)) for n in symbols] # deprecated
@@ -505,12 +518,15 @@ class CoordSystem(Basic):
         return result
 
     def connect_to(self, to_sys, from_coords, to_exprs, inverse=True, fill_in_gaps=False):
-        SymPyDeprecationWarning(
-            feature="CoordSystem.connect_to",
-            useinstead="new instance generated with new 'transforms' parameter",
-            issue=19321,
-            deprecated_since_version="1.7"
-        ).warn()
+        sympy_deprecation_warning(
+            """
+            The CoordSystem.connect_to() method is deprecated. Instead,
+            generate a new instance of CoordSystem with the 'relations'
+            keyword argument (CoordSystem classes are now immutable).
+            """,
+            deprecated_since_version="1.7",
+            active_deprecations_target="deprecated-diffgeom-mutable",
+        )
 
         from_coords, to_exprs = dummyfy(from_coords, to_exprs)
         self.transforms[to_sys] = Matrix(from_coords), Matrix(to_exprs)
@@ -582,16 +598,19 @@ class CoordSystem(Basic):
 
     def coord_tuple_transform_to(self, to_sys, coords):
         """Transform ``coords`` to coord system ``to_sys``."""
-        SymPyDeprecationWarning(
-            feature="CoordSystem.coord_tuple_transform_to",
-            useinstead="CoordSystem.transform",
-            issue=19321,
-            deprecated_since_version="1.7"
-        ).warn()
+        sympy_deprecation_warning(
+            """
+            The CoordSystem.coord_tuple_transform_to() method is deprecated.
+            Use the CoordSystem.transform() method instead.
+            """,
+            deprecated_since_version="1.7",
+            active_deprecations_target="deprecated-diffgeom-mutable",
+        )
 
         coords = Matrix(coords)
         if self != to_sys:
-            transf = self.transforms[to_sys]
+            with ignore_warnings(SymPyDeprecationWarning):
+                transf = self.transforms[to_sys]
             coords = transf[1].subs(list(zip(transf[0], coords)))
         return coords
 
@@ -972,10 +991,7 @@ class BaseScalarField(Expr):
         return simplify(coords[self._index]).doit()
 
     # XXX Workaround for limitations on the content of args
-    free_symbols = set()  # type: Set[Any]
-
-    def doit(self):
-        return self
+    free_symbols = set()  # type: tSet[Any]
 
 
 class BaseVectorField(Expr):
@@ -1127,7 +1143,7 @@ class Commutator(Expr):
 
     >>> from sympy.diffgeom.rn import R2_p, R2_r
     >>> from sympy.diffgeom import Commutator
-    >>> from sympy.simplify import simplify
+    >>> from sympy import simplify
 
     >>> fx, fy = R2_r.base_scalars()
     >>> e_x, e_y = R2_r.base_vectors()
@@ -1635,6 +1651,7 @@ class CovarDerivativeOp(Expr):
             raise ValueError('Covariant derivatives are defined only with '
                              'respect to vector fields. The supplied argument '
                              'was not a vector field.')
+        christoffel = ImmutableDenseNDimArray(christoffel)
         obj = super().__new__(cls, wrt, christoffel)
         # deprecated assigments
         obj._wrt = wrt
@@ -2074,8 +2091,7 @@ def metric_to_Christoffel_1st(expr):
         raise ValueError(
             'The two-form representing the metric is not symmetric.')
     coord_sys = _find_coords(expr).pop()
-    deriv_matrices = [matrix.applyfunc(lambda a: d(a))
-                      for d in coord_sys.base_vectors()]
+    deriv_matrices = [matrix.applyfunc(d) for d in coord_sys.base_vectors()]
     indices = list(range(coord_sys.dim))
     christoffel = [[[(deriv_matrices[k][i, j] + deriv_matrices[j][i, k] - deriv_matrices[i][j, k])/2
                      for k in indices]
@@ -2219,19 +2235,17 @@ class _deprecated_container:
     # This class gives deprecation warning.
     # When deprecated features are completely deleted, this should be removed as well.
     # See https://github.com/sympy/sympy/pull/19368
-    def __init__(self, feature, useinstead, issue, version, data):
+    def __init__(self, message, data):
         super().__init__(data)
-        self.feature = feature
-        self.useinstead = useinstead
-        self.issue = issue
-        self.version = version
+        self.message = message
 
     def warn(self):
-        SymPyDeprecationWarning(
-                    feature=self.feature,
-                    useinstead=self.useinstead,
-                    issue=self.issue,
-                    deprecated_since_version=self.version).warn()
+        sympy_deprecation_warning(
+            self.message,
+            deprecated_since_version="1.7",
+            active_deprecations_target="deprecated-diffgeom-mutable",
+            stacklevel=4
+        )
 
     def __iter__(self):
         self.warn()
@@ -2245,8 +2259,14 @@ class _deprecated_container:
         self.warn()
         return super().__contains__(key)
 
+
 class _deprecated_list(_deprecated_container, list):
     pass
 
+
 class _deprecated_dict(_deprecated_container, dict):
     pass
+
+
+# Import at end to avoid cyclic imports
+from sympy.simplify.simplify import simplify
