@@ -1,31 +1,33 @@
-from sympy.core import Function, S, sympify
+from sympy.core import Function, S, sympify, NumberKind
 from sympy.utilities.iterables import sift
 from sympy.core.add import Add
 from sympy.core.containers import Tuple
-from sympy.core.compatibility import ordered
 from sympy.core.operations import LatticeOp, ShortCircuit
 from sympy.core.function import (Application, Lambda,
     ArgumentIndexError)
 from sympy.core.expr import Expr
+from sympy.core.exprtools import factor_terms
 from sympy.core.mod import Mod
 from sympy.core.mul import Mul
 from sympy.core.numbers import Rational
 from sympy.core.power import Pow
 from sympy.core.relational import Eq, Relational
 from sympy.core.singleton import Singleton
+from sympy.core.sorting import ordered
 from sympy.core.symbol import Dummy
 from sympy.core.rules import Transform
 from sympy.core.logic import fuzzy_and, fuzzy_or, _torf
+from sympy.core.traversal import walk
+from sympy.core.numbers import Integer
 from sympy.logic.boolalg import And, Or
 
+
 def _minmax_as_Piecewise(op, *args):
     # helper for Min/Max rewrite as Piecewise
     from sympy.functions.elementary.piecewise import Piecewise
     ec = []
     for i, a in enumerate(args):
-        c = []
-        for j in range(i + 1, len(args)):
-            c.append(Relational(a, args[j], op))
+        c = [Relational(a, args[j], op) for j in range(i + 1, len(args))]
         ec.append((a, And(*c)))
     return Piecewise(*ec)
 
@@ -128,9 +130,7 @@ def sqrt(arg, evaluate=None):
     >>> func_name(sqrt(x))
     'Pow'
     >>> sqrt(x).has(sqrt)
-    Traceback (most recent call last):
-      ...
-    sympy.core.sympify.SympifyError: SympifyError: <function sqrt at 0x10e8900d0>
+    False
 
     To find ``sqrt`` look for ``Pow`` with an exponent of ``1/2``:
 
@@ -202,8 +202,8 @@ def cbrt(arg, evaluate=None):
     References
     ==========
 
-    * https://en.wikipedia.org/wiki/Cube_root
-    * https://en.wikipedia.org/wiki/Principal_value
+    .. [1] https://en.wikipedia.org/wiki/Cube_root
+    .. [2] https://en.wikipedia.org/wiki/Principal_value
 
     """
     return Pow(arg, Rational(1, 3), evaluate=evaluate)
@@ -296,11 +296,11 @@ def root(arg, n, k=0, evaluate=None):
     References
     ==========
 
-    * https://en.wikipedia.org/wiki/Square_root
-    * https://en.wikipedia.org/wiki/Real_root
-    * https://en.wikipedia.org/wiki/Root_of_unity
-    * https://en.wikipedia.org/wiki/Principal_value
-    * http://mathworld.wolfram.com/CubeRoot.html
+    .. [1] https://en.wikipedia.org/wiki/Square_root
+    .. [2] https://en.wikipedia.org/wiki/Real_root
+    .. [3] https://en.wikipedia.org/wiki/Root_of_unity
+    .. [4] https://en.wikipedia.org/wiki/Principal_value
+    .. [5] http://mathworld.wolfram.com/CubeRoot.html
 
     """
     n = sympify(n)
@@ -310,14 +310,14 @@ def root(arg, n, k=0, evaluate=None):
 
 
 def real_root(arg, n=None, evaluate=None):
-    """Return the real *n*'th-root of *arg* if possible.
+    r"""Return the real *n*'th-root of *arg* if possible.
 
     Parameters
     ==========
 
     n : int or None, optional
         If *n* is ``None``, then all instances of
-        ``(-n)**(1/odd)`` will be changed to ``-n**(1/odd)``.
+        $(-n)^{1/\text{odd}}$ will be changed to $-n^{1/\text{odd}}$.
         This will only create a real root of a principal root.
         The presence of other factors may cause the result to not be
         real.
@@ -378,7 +378,8 @@ def real_root(arg, n=None, evaluate=None):
 
 class MinMaxBase(Expr, LatticeOp):
     def __new__(cls, *args, **assumptions):
-        evaluate = assumptions.pop('evaluate', True)
+        from sympy.core.parameters import global_parameters
+        evaluate = assumptions.pop('evaluate', global_parameters.evaluate)
         args = (sympify(arg) for arg in args)
 
         # first standard filter, for cls.zero and cls.identity
@@ -389,14 +390,11 @@ class MinMaxBase(Expr, LatticeOp):
                 args = frozenset(cls._new_args_filter(args))
             except ShortCircuit:
                 return cls.zero
-        else:
-            args = frozenset(args)
-
-        if evaluate:
             # remove redundant args that are easily identified
             args = cls._collapse_arguments(args, **assumptions)
             # find local zeros
             args = cls._find_localzeros(args, **assumptions)
+        args = frozenset(args)
 
         if not args:
             return cls.identity
@@ -405,9 +403,8 @@ class MinMaxBase(Expr, LatticeOp):
             return list(args).pop()
 
         # base creation
-        _args = frozenset(args)
-        obj = Expr.__new__(cls, *ordered(_args), **assumptions)
-        obj._argset = _args
+        obj = Expr.__new__(cls, *ordered(args), **assumptions)
+        obj._argset = args
         return obj
 
     @classmethod
@@ -434,9 +431,6 @@ class MinMaxBase(Expr, LatticeOp):
         >>> Min(a, Max(b, Min(c, d, Max(a, e))))
         Min(a, Max(b, Min(a, c, d)))
         """
-        from sympy.utilities.iterables import ordered
-        from sympy.simplify.simplify import walk
-
         if not args:
             return args
         args = list(ordered(args))
@@ -469,17 +463,17 @@ class MinMaxBase(Expr, LatticeOp):
             # local zeros have not been handled yet, so look through
             # more than the first arg
             if cls == Min:
-                for i in range(len(args)):
-                    if not args[i].is_number:
+                for arg in args:
+                    if not arg.is_number:
                         break
-                    if (args[i] < small) == True:
-                        small = args[i]
+                    if (arg < small) == True:
+                        small = arg
             elif cls == Max:
-                for i in range(len(args)):
-                    if not args[i].is_number:
+                for arg in args:
+                    if not arg.is_number:
                         break
-                    if (args[i] > big) == True:
-                        big = args[i]
+                    if (arg > big) == True:
+                        big = arg
             T = None
             if cls == Min:
                 if small != Min.identity:
@@ -554,11 +548,10 @@ class MinMaxBase(Expr, LatticeOp):
         Generator filtering args.
 
         first standard filter, for cls.zero and cls.identity.
-        Also reshape Max(a, Max(b, c)) to Max(a, b, c),
+        Also reshape ``Max(a, Max(b, c))`` to ``Max(a, b, c)``,
         and check arguments for comparability
         """
         for arg in arg_sequence:
-
             # pre-filter, checking comparability of arguments
             if not isinstance(arg, Expr) or arg.is_extended_real is False or (
                     arg.is_number and
@@ -606,25 +599,24 @@ class MinMaxBase(Expr, LatticeOp):
         """
         Check if x and y are connected somehow.
         """
-        from sympy.core.exprtools import factor_terms
-        def hit(v, t, f):
-            if not v.is_Relational:
-                return t if v else f
         for i in range(2):
             if x == y:
                 return True
-            r = hit(x >= y, Max, Min)
-            if r is not None:
-                return r
-            r = hit(y <= x, Max, Min)
-            if r is not None:
-                return r
-            r = hit(x <= y, Min, Max)
-            if r is not None:
-                return r
-            r = hit(y >= x, Min, Max)
-            if r is not None:
-                return r
+            t, f = Max, Min
+            for op in "><":
+                for j in range(2):
+                    try:
+                        if op == ">":
+                            v = x >= y
+                        else:
+                            v = x <= y
+                    except TypeError:
+                        return False  # non-real arg
+                    if not v.is_Relational:
+                        return t if v else f
+                    t, f = f, t
+                    x, y = y, x
+                x, y = y, x  # run next pass with reversed order relative to start
             # simplification can be expensive, so be conservative
             # in what is attempted
             x = factor_terms(x - y)
@@ -687,15 +679,16 @@ class MinMaxBase(Expr, LatticeOp):
     _eval_is_transcendental = lambda s: _torf(i.is_transcendental for i in s.args)
     _eval_is_zero = lambda s: _torf(i.is_zero for i in s.args)
 
+
 class Max(MinMaxBase, Application):
-    """
+    r"""
     Return, if possible, the maximum value of the list.
 
     When number of arguments is equal one, then
     return this argument.
 
     When number of arguments is equal two, then
-    return, if possible, the value from (a, b) that is >= the other.
+    return, if possible, the value from (a, b) that is $\ge$ the other.
 
     In common case, when the length of list greater than 2, the task
     is more complicated. Return only the arguments, which are greater
@@ -753,13 +746,13 @@ class Max(MinMaxBase, Application):
     symbol `x` with negative assumption is comparable with a natural number.
 
     Also there are "least" elements, which are comparable with all others,
-    and have a zero property (maximum or minimum for all elements). E.g. `oo`.
-    In case of it the allocation operation is terminated and only this value is
-    returned.
+    and have a zero property (maximum or minimum for all elements).
+    For example, in case of $\infty$, the allocation operation is terminated
+    and only this value is returned.
 
     Assumption:
-       - if A > B > C then A > C
-       - if A == B then B can be removed
+       - if $A > B > C$ then $A > C$
+       - if $A = B$ then $B$ can be removed
 
     References
     ==========
@@ -776,7 +769,7 @@ class Max(MinMaxBase, Application):
     identity = S.NegativeInfinity
 
     def fdiff( self, argindex ):
-        from sympy import Heaviside
+        from sympy.functions.special.delta_functions import Heaviside
         n = len(self.args)
         if 0 < argindex and argindex <= n:
             argindex -= 1
@@ -788,7 +781,7 @@ class Max(MinMaxBase, Application):
             raise ArgumentIndexError(self, argindex)
 
     def _eval_rewrite_as_Heaviside(self, *args, **kwargs):
-        from sympy import Heaviside
+        from sympy.functions.special.delta_functions import Heaviside
         return Add(*[j*Mul(*[Heaviside(j - i) for i in args if i!=j]) \
                 for j in args])
 
@@ -839,7 +832,7 @@ class Min(MinMaxBase, Application):
     identity = S.Infinity
 
     def fdiff( self, argindex ):
-        from sympy import Heaviside
+        from sympy.functions.special.delta_functions import Heaviside
         n = len(self.args)
         if 0 < argindex and argindex <= n:
             argindex -= 1
@@ -851,7 +844,7 @@ class Min(MinMaxBase, Application):
             raise ArgumentIndexError(self, argindex)
 
     def _eval_rewrite_as_Heaviside(self, *args, **kwargs):
-        from sympy import Heaviside
+        from sympy.functions.special.delta_functions import Heaviside
         return Add(*[j*Mul(*[Heaviside(i-j) for i in args if i!=j]) \
                 for j in args])
 
@@ -866,3 +859,57 @@ class Min(MinMaxBase, Application):
 
     def _eval_is_negative(self):
         return fuzzy_or(a.is_negative for a in self.args)
+
+
+class Rem(Function):
+    """Returns the remainder when ``p`` is divided by ``q`` where ``p`` is finite
+    and ``q`` is not equal to zero. The result, ``p - int(p/q)*q``, has the same sign
+    as the divisor.
+
+    Parameters
+    ==========
+
+    p : Expr
+        Dividend.
+
+    q : Expr
+        Divisor.
+
+    Notes
+    =====
+
+    ``Rem`` corresponds to the ``%`` operator in C.
+
+    Examples
+    ========
+
+    >>> from sympy.abc import x, y
+    >>> from sympy import Rem
+    >>> Rem(x**3, y)
+    Rem(x**3, y)
+    >>> Rem(x**3, y).subs({x: -5, y: 3})
+    -2
+
+    See Also
+    ========
+
+    Mod
+    """
+    kind = NumberKind
+
+    @classmethod
+    def eval(cls, p, q):
+        """ the function remainder if both p,q are numbers
+            and q is not zero
+        """
+
+        if q.is_zero:
+            raise ZeroDivisionError("Division by zero")
+        if p is S.NaN or q is S.NaN or p.is_finite is False or q.is_finite is False:
+            return S.NaN
+        if p is S.Zero or p in (q, -q) or (p.is_integer and q == 1):
+            return S.Zero
+
+        if q.is_Number:
+            if p.is_Number:
+                return p - Integer(p/q)*q
