@@ -1,8 +1,15 @@
-from sympy import Rational, oo, sqrt, S
-from sympy import Line, Point, Point2D, Parabola, Segment2D, Ray2D
-from sympy import Circle, Ellipse, symbols, sign
+from sympy.core.numbers import (Rational, oo)
+from sympy.core.singleton import S
+from sympy.core.symbol import symbols
+from sympy.functions.elementary.complexes import sign
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.geometry.ellipse import (Circle, Ellipse)
+from sympy.geometry.line import (Line, Ray2D, Segment2D)
+from sympy.geometry.parabola import Parabola
+from sympy.geometry.point import (Point, Point2D)
 from sympy.testing.pytest import raises
 
+from sympy.abc import x, y
 
 def test_parabola_geom():
     a, b = symbols('a b')
@@ -31,11 +38,12 @@ def test_parabola_geom():
     pa9 = Parabola(p4, d3)
     pa10 = Parabola(p5, d5)
     pa11 = Parabola(p5, d6)
+    d = Line(Point(3, 7), Point(2, 9))
+    pa12 = Parabola(Point(7, 8), d)
+    pa12r = Parabola(Point(7, 8).reflect(d), d)
 
     raises(ValueError, lambda:
            Parabola(Point(7, 8, 9), Line(Point(6, 7), Point(7, 7))))
-    raises(NotImplementedError, lambda:
-           Parabola(Point(7, 8), Line(Point(3, 7), Point(2, 9))))
     raises(ValueError, lambda:
            Parabola(Point(0, 2), Line(Point(7, 2), Point(6, 2))))
     raises(ValueError, lambda: Parabola(Point(7, 8), Point(3, 8)))
@@ -72,6 +80,20 @@ def test_parabola_geom():
     assert pa10.focal_length == pa11.focal_length == sqrt((a - b) ** 2) / 2 # if a, b real == abs(a - b)/2
     assert pa11.vertex == Point(*pa10.vertex[::-1]) == Point(a,
                             a - sqrt((a - b)**2)*sign(a - b)/2) # change axis x->y, y->x on pa10
+    aos = pa12.axis_of_symmetry
+    assert aos == Line(Point(7, 8), Point(5, 7))
+    assert pa12.directrix == Line(Point(3, 7), Point(2, 9))
+    assert pa12.directrix.angle_between(aos) == S.Pi/2
+    assert pa12.eccentricity == 1
+    assert pa12.equation(x, y) == (x - 7)**2 + (y - 8)**2 - (-2*x - y + 13)**2/5
+    assert pa12.focal_length == 9*sqrt(5)/10
+    assert pa12.focus == Point(7, 8)
+    assert pa12.p_parameter == 9*sqrt(5)/10
+    assert pa12.vertex == Point2D(S(26)/5, S(71)/10)
+    assert pa12r.focal_length == 9*sqrt(5)/10
+    assert pa12r.focus == Point(-S(1)/5, S(22)/5)
+    assert pa12r.p_parameter == -9*sqrt(5)/10
+    assert pa12r.vertex == Point(S(8)/5, S(53)/10)
 
 
 def test_parabola_intersection():
