@@ -14,7 +14,7 @@ right hand side of the equation (i.e., gi in k(t)), and Q is a list of terms on
 the right hand side of the equation (i.e., qi in k[t]).  See the docstring of
 each function for more information.
 """
-
+import itertools
 from functools import reduce
 
 from sympy.core import Dummy, ilcm, Add, Mul, Pow, S
@@ -118,7 +118,7 @@ def prde_special_denom(a, ba, bd, G, DE, case='auto'):
         p = Poly(DE.t, DE.t)
     elif case == 'tan':
         p = Poly(DE.t**2 + 1, DE.t)
-    elif case in ['primitive', 'base']:
+    elif case in ('primitive', 'base'):
         B = ba.quo(bd)
         return (a, B, G, Poly(1, DE.t))
     else:
@@ -185,7 +185,7 @@ def prde_linear_constraints(a, b, G, DE):
     (c1, ..., cm) is a solution of Mx == 0, and p and the ci satisfy
     a*Dp + b*p == Sum(ci*qi, (i, 1, m)).
 
-    Because M has entries in k(t), and because Matrix doesn't play well with
+    Because M has entries in k(t), and because Matrix does not play well with
     Poly, M will be a Matrix of Basic expressions.
     """
     m = len(G)
@@ -195,8 +195,8 @@ def prde_linear_constraints(a, b, G, DE):
     d = Poly(d, field=True)
     Q = [(ga*(d).quo(gd)).div(d) for ga, gd in G]
 
-    if not all([ri.is_zero for _, ri in Q]):
-        N = max([ri.degree(DE.t) for _, ri in Q])
+    if not all(ri.is_zero for _, ri in Q):
+        N = max(ri.degree(DE.t) for _, ri in Q)
         M = Matrix(N + 1, m, lambda i, j: Q[j][1].nth(i), DE.t)
     else:
         M = Matrix(0, m, [], DE.t)  # No constraints, return the empty matrix.
@@ -215,8 +215,8 @@ def poly_linear_constraints(p, d):
     m = len(p)
     q, r = zip(*[pi.div(d) for pi in p])
 
-    if not all([ri.is_zero for ri in r]):
-        n = max([ri.degree() for ri in r])
+    if not all(ri.is_zero for ri in r):
+        n = max(ri.degree() for ri in r)
         M = Matrix(n + 1, m, lambda i, j: r[j].nth(i), d.gens)
     else:
         M = Matrix(0, m, [], d.gens)  # No constraints.
@@ -274,22 +274,21 @@ def constant_system(A, u, DE):
 
     D = lambda x: derivation(x, DE, basic=True)
 
-    for j in range(A.cols):
-        for i in range(A.rows):
-            if A[i, j].expr.has(*DE.T):
-                # This assumes that const(F(t0, ..., tn) == const(K) == F
-                Ri = A[i, :]
-                # Rm+1; m = A.rows
-                DAij = D(A[i, j])
-                Rm1 = Ri.applyfunc(lambda x: D(x) / DAij)
-                um1 = D(u[i]) / DAij
+    for j, i in itertools.product(range(A.cols), range(A.rows)):
+        if A[i, j].expr.has(*DE.T):
+            # This assumes that const(F(t0, ..., tn) == const(K) == F
+            Ri = A[i, :]
+            # Rm+1; m = A.rows
+            DAij = D(A[i, j])
+            Rm1 = Ri.applyfunc(lambda x: D(x) / DAij)
+            um1 = D(u[i]) / DAij
 
-                Aj = A[:, j]
-                A = A - Aj * Rm1
-                u = u - Aj * um1
+            Aj = A[:, j]
+            A = A - Aj * Rm1
+            u = u - Aj * um1
 
-                A = A.col_join(Rm1)
-                u = u.col_join(Matrix([um1], u.gens))
+            A = A.col_join(Rm1)
+            u = u.col_join(Matrix([um1], u.gens))
 
     return (A, u)
 
@@ -337,12 +336,11 @@ def prde_no_cancel_b_large(b, Q, n, DE):
     m = len(Q)
     H = [Poly(0, DE.t)]*m
 
-    for N in range(n, -1, -1):  # [n, ..., 0]
-        for i in range(m):
-            si = Q[i].nth(N + db)/b.LC()
-            sitn = Poly(si*DE.t**N, DE.t)
-            H[i] = H[i] + sitn
-            Q[i] = Q[i] - derivation(sitn, DE) - b*sitn
+    for N, i in itertools.product(range(n, -1, -1), range(m)):  # [n, ..., 0]
+        si = Q[i].nth(N + db)/b.LC()
+        sitn = Poly(si*DE.t**N, DE.t)
+        H[i] = H[i] + sitn
+        Q[i] = Q[i] - derivation(sitn, DE) - b*sitn
 
     if all(qi.is_zero for qi in Q):
         dc = -1
@@ -374,12 +372,11 @@ def prde_no_cancel_b_small(b, Q, n, DE):
     m = len(Q)
     H = [Poly(0, DE.t)]*m
 
-    for N in range(n, 0, -1):  # [n, ..., 1]
-        for i in range(m):
-            si = Q[i].nth(N + DE.d.degree(DE.t) - 1)/(N*DE.d.LC())
-            sitn = Poly(si*DE.t**N, DE.t)
-            H[i] = H[i] + sitn
-            Q[i] = Q[i] - derivation(sitn, DE) - b*sitn
+    for N, i in itertools.product(range(n, 0, -1), range(m)):  # [n, ..., 1]
+        si = Q[i].nth(N + DE.d.degree(DE.t) - 1)/(N*DE.d.LC())
+        sitn = Poly(si*DE.t**N, DE.t)
+        H[i] = H[i] + sitn
+        Q[i] = Q[i] - derivation(sitn, DE) - b*sitn
 
     if b.degree(DE.t) > 0:
         for i in range(m):
@@ -525,7 +522,7 @@ def param_poly_rischDE(a, b, q, n, DE):
     if n < 0:
         # Only the trivial zero solution is possible.
         # Find relations between the qi.
-        if all([qi.is_zero for qi in q]):
+        if all(qi.is_zero for qi in q):
             return [], zeros(1, m, DE.t)  # No constraints.
 
         N = max([qi.degree(DE.t) for qi in q])
@@ -555,7 +552,7 @@ def param_poly_rischDE(a, b, q, n, DE):
 
         else:
             # Liouvillian cases
-            if DE.case == 'primitive' or DE.case == 'exp':
+            if DE.case in ('primitive', 'exp'):
                 return prde_cancel_liouvillian(b, q, n, DE)
             else:
                 raise NotImplementedError("non-linear and hypertangent "
@@ -763,8 +760,14 @@ def param_rischDE(fa, fd, G, DE):
     # y = Sum(blk'*hk, (k, 1, v))/gamma, where k' = k + m + u.
 
     v = len(h)
-    M = Matrix([wl[:m] + wl[-v:] for wl in W])  # excise dj's.
+    shape = (len(W), m+v)
+    elements = [wl[:m] + wl[-v:] for wl in W] # excise dj's.
+    items = [e for row in elements for e in row]
+
+    # Need to set the shape in case W is empty
+    M = Matrix(*shape, items, DE.t)
     N = M.nullspace()
+
     # N = [n1, ..., ns] where the ni in Const(k)^(m + v) are column
     # vectors generating the space of linear relations between
     # c1, ..., cm, e1, ..., ev.
@@ -806,7 +809,7 @@ def limited_integrate_reduce(fa, fd, G, DE):
 
     # These are the cases where we know that S1irr = Sirr, but there could be
     # others, and this algorithm will need to be extended to handle them.
-    if DE.case in ['base', 'primitive', 'exp', 'tan']:
+    if DE.case in ('base', 'primitive', 'exp', 'tan'):
         hs = reduce(lambda i, j: i.lcm(j), (ds,) + Es)  # lcm(ds, es1, ..., esm)
         a = hn*hs
         b -= (hn*derivation(hs, DE)).quo(hs)
@@ -1047,7 +1050,7 @@ def is_deriv_k(fa, fd, DE):
 
     u = u.to_Matrix()  # Poly to Expr
 
-    if not all(derivation(i, DE, basic=True).is_zero for i in u) or not A:
+    if not A or not all(derivation(i, DE, basic=True).is_zero for i in u):
         # If the elements of u are not all constant
         # Note: See comment in constant_system
 
@@ -1172,7 +1175,7 @@ def is_log_deriv_k_t_radical(fa, fd, DE, Df=True):
 
     u = u.to_Matrix()  # Poly to Expr
 
-    if not all(derivation(i, DE, basic=True).is_zero for i in u) or not A:
+    if not A or not all(derivation(i, DE, basic=True).is_zero for i in u):
         # If the elements of u are not all constant
         # Note: See comment in constant_system
 
@@ -1312,7 +1315,7 @@ def is_log_deriv_k_t_radical_in_field(fa, fd, DE, case='auto', z=None):
         raise NotImplementedError("The hypertangent case is "
         "not yet implemented for is_log_deriv_k_t_radical_in_field()")
 
-    elif case in ['other_linear', 'other_nonlinear']:
+    elif case in ('other_linear', 'other_nonlinear'):
         # XXX: If these are supported by the structure theorems, change to NotImplementedError.
         raise ValueError("The %s case is not supported in this function." % case)
 
