@@ -1,11 +1,13 @@
 """Tools for setting up interactive sessions. """
 
+from sympy.external.gmpy import GROUND_TYPES
 from sympy.external.importtools import version_tuple
 
 from sympy.interactive.printing import init_printing
 
+from sympy.utilities.misc import ARCH
+
 preexec_source = """\
-from __future__ import division
 from sympy import *
 x, y, z, t = symbols('x y z t')
 k, m, n = symbols('k m n', integer=True)
@@ -20,7 +22,7 @@ Documentation can be found at https://docs.sympy.org/%(version)s
 """
 
 no_ipython = """\
-Couldn't locate IPython. Having IPython installed is greatly recommended.
+Could not locate IPython. Having IPython installed is greatly recommended.
 See http://ipython.scipy.org for more details. If you use Debian/Ubuntu,
 just install the 'ipython' package and start isympy again.
 """
@@ -29,8 +31,6 @@ just install the 'ipython' package and start isympy again.
 def _make_message(ipython=True, quiet=False, source=None):
     """Create a banner for an interactive session. """
     from sympy import __version__ as sympy_version
-    from sympy.external.gmpy import GROUND_TYPES
-    from sympy.utilities.misc import ARCH
     from sympy import SYMPY_DEBUG
 
     import sys
@@ -94,7 +94,6 @@ def int_to_Integer(s):
     Examples
     ========
 
-    >>> from __future__ import division
     >>> from sympy import Integer # noqa: F401
     >>> from sympy.interactive.session import int_to_Integer
     >>> s = '1.2 + 1/2 - 0x12 + a1'
@@ -271,7 +270,7 @@ def init_python_session():
         """An interactive console with readline support. """
 
         def __init__(self):
-            ns_locals = dict()
+            ns_locals = {}
             InteractiveConsole.__init__(self, locals=ns_locals)
             try:
                 import rlcompleter
