@@ -1,10 +1,19 @@
 """Quantum mechanical angular momemtum."""
 
-from __future__ import print_function, division
-
-from sympy import (Add, binomial, cos, exp, Expr, factorial, I, Integer, Mul,
-                   pi, Rational, S, sin, simplify, sqrt, Sum, symbols, sympify,
-                   Tuple, Dummy)
+from sympy.concrete.summations import Sum
+from sympy.core.add import Add
+from sympy.core.containers import Tuple
+from sympy.core.expr import Expr
+from sympy.core.mul import Mul
+from sympy.core.numbers import (I, Integer, Rational, pi)
+from sympy.core.singleton import S
+from sympy.core.symbol import (Dummy, symbols)
+from sympy.core.sympify import sympify
+from sympy.functions.combinatorial.factorials import (binomial, factorial)
+from sympy.functions.elementary.exponential import exp
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.trigonometric import (cos, sin)
+from sympy.simplify.simplify import simplify
 from sympy.matrices import zeros
 from sympy.printing.pretty.stringpict import prettyForm, stringPict
 from sympy.printing.pretty.pretty_symbology import pretty_symbol
@@ -65,7 +74,7 @@ def m_values(j):
 #-----------------------------------------------------------------------------
 
 
-class SpinOpBase(object):
+class SpinOpBase:
     """Base class for spin operators."""
 
     @classmethod
@@ -133,7 +142,7 @@ class SpinOpBase(object):
     def _apply_operator_TensorProduct(self, tp, **options):
         # Uncoupling operator is only easily found for coordinate basis spin operators
         # TODO: add methods for uncoupling operators
-        if not (isinstance(self, JxOp) or isinstance(self, JyOp) or isinstance(self, JzOp)):
+        if not isinstance(self, (JxOp, JyOp, JzOp)):
             raise NotImplementedError
         result = []
         for n in range(len(tp.args)):
@@ -852,12 +861,14 @@ class WignerD(Expr):
         return WignerD(*self.args, **hints)
 
     def _eval_wignerd(self):
-        j = sympify(self.j)
-        m = sympify(self.m)
-        mp = sympify(self.mp)
-        alpha = sympify(self.alpha)
-        beta = sympify(self.beta)
-        gamma = sympify(self.gamma)
+        j = self.j
+        m = self.m
+        mp = self.mp
+        alpha = self.alpha
+        beta = self.beta
+        gamma = self.gamma
+        if alpha == 0 and beta == 0 and gamma == 0:
+            return KroneckerDelta(m, mp)
         if not j.is_number:
             raise ValueError(
                 'j parameter must be numerical to evaluate, got %s' % j)
@@ -951,7 +962,6 @@ class SpinState(State):
         gamma = sympify(options.get('gamma', 0))
         size, mvals = m_values(j)
         result = zeros(size, 1)
-        # TODO: Use KroneckerDelta if all Euler angles == 0
         # breaks finding angles on L930
         for p, mval in enumerate(mvals):
             if m.is_number:
@@ -1360,13 +1370,13 @@ class CoupledSpinState(SpinState):
         else:
             raise TypeError("CoupledSpinState only takes 3 or 4 arguments, got: %s" % (len(jcoupling) + 3) )
         # Check arguments have correct form
-        if not (isinstance(jn, list) or isinstance(jn, tuple) or isinstance(jn, Tuple)):
+        if not isinstance(jn, (list, tuple, Tuple)):
             raise TypeError('jn must be Tuple, list or tuple, got %s' %
                             jn.__class__.__name__)
-        if not (isinstance(jcoupling, list) or isinstance(jcoupling, tuple) or isinstance(jcoupling, Tuple)):
+        if not isinstance(jcoupling, (list, tuple, Tuple)):
             raise TypeError('jcoupling must be Tuple, list or tuple, got %s' %
                             jcoupling.__class__.__name__)
-        if not all(isinstance(term, list) or isinstance(term, tuple) or isinstance(term, Tuple) for term in jcoupling):
+        if not all(isinstance(term, (list, tuple, Tuple)) for term in jcoupling):
             raise TypeError(
                 'All elements of jcoupling must be list, tuple or Tuple')
         if not len(jn) - 1 == len(jcoupling):
@@ -1845,10 +1855,10 @@ def couple(expr, jcoupling_list=None):
     a = expr.atoms(TensorProduct)
     for tp in a:
         # Allow other tensor products to be in expression
-        if not all([ isinstance(state, SpinState) for state in tp.args]):
+        if not all(isinstance(state, SpinState) for state in tp.args):
             continue
         # If tensor product has all spin states, raise error for invalid tensor product state
-        if not all([state.__class__ is tp.args[0].__class__ for state in tp.args]):
+        if not all(state.__class__ is tp.args[0].__class__ for state in tp.args):
             raise TypeError('All states must be the same basis')
         expr = expr.subs(tp, _couple(tp, jcoupling_list))
     return expr
@@ -1870,9 +1880,9 @@ def _couple(tp, jcoupling_list):
                         (len(states) - 1, len(jcoupling_list)))
     if not all( len(coupling) == 2 for coupling in jcoupling_list):
         raise ValueError('Each coupling must define 2 spaces')
-    if any([n1 == n2 for n1, n2 in jcoupling_list]):
+    if any(n1 == n2 for n1, n2 in jcoupling_list):
         raise ValueError('Spin spaces cannot couple to themselves')
-    if all([sympify(n1).is_number and sympify(n2).is_number for n1, n2 in jcoupling_list]):
+    if all(sympify(n1).is_number and sympify(n2).is_number for n1, n2 in jcoupling_list):
         j_test = [0]*len(states)
         for n1, n2 in jcoupling_list:
             if j_test[n1 - 1] == -1 or j_test[n2 - 1] == -1:
@@ -1913,7 +1923,7 @@ def _couple(tp, jcoupling_list):
 
                 # Skip the configuration if non-physical
                 # This is a lazy check for physical states given the loose restrictions of diff_max
-                if any( [ d > m for d, m in zip(diff_list, diff_max) ] ):
+                if any(d > m for d, m in zip(diff_list, diff_max)):
                     continue
 
                 # Determine term
@@ -1931,11 +1941,11 @@ def _couple(tp, jcoupling_list):
                     cg_terms.append( (j1, m1, j2, m2, j3, m3) )
                     jcoupling.append( (min(j1_n), min(j2_n), j3) )
                 # Better checks that state is physical
-                if any([ abs(term[5]) > term[4] for term in cg_terms ]):
+                if any(abs(term[5]) > term[4] for term in cg_terms):
                     continue
-                if any([ term[0] + term[2] < term[4] for term in cg_terms ]):
+                if any(term[0] + term[2] < term[4] for term in cg_terms):
                     continue
-                if any([ abs(term[0] - term[2]) > term[4] for term in cg_terms ]):
+                if any(abs(term[0] - term[2]) > term[4] for term in cg_terms):
                     continue
                 coeff = Mul( *[ CG(*term).doit() for term in cg_terms] )
                 state = coupled_evect(j3, m3, jn, jcoupling)
@@ -2050,7 +2060,7 @@ def _uncouple(state, jn, jcoupling_list):
     elif isinstance(state, SpinState):
         if jn is None:
             raise ValueError("Must specify j-values for coupled state")
-        if not (isinstance(jn, list) or isinstance(jn, tuple)):
+        if not isinstance(jn, (list, tuple)):
             raise TypeError("jn must be list or tuple")
         if jcoupling_list is None:
             # Use default
@@ -2058,7 +2068,7 @@ def _uncouple(state, jn, jcoupling_list):
             for i in range(1, len(jn)):
                 jcoupling_list.append(
                     (1, 1 + i, Add(*[jn[j] for j in range(i + 1)])) )
-        if not (isinstance(jcoupling_list, list) or isinstance(jcoupling_list, tuple)):
+        if not isinstance(jcoupling_list, (list, tuple)):
             raise TypeError("jcoupling must be a list or tuple")
         if not len(jcoupling_list) == len(jn) - 1:
             raise ValueError("Must specify 2 fewer coupling terms than the number of j values")
@@ -2091,7 +2101,7 @@ def _uncouple(state, jn, jcoupling_list):
         result = []
         for config_num in range(tot):
             diff_list = _confignum_to_difflist(config_num, diff, n)
-            if any( [ d > p for d, p in zip(diff_list, diff_max) ] ):
+            if any(d > p for d, p in zip(diff_list, diff_max)):
                 continue
 
             cg_terms = []
