@@ -1,6 +1,9 @@
 """Implementation of :class:`AlgebraicField` class. """
 
 
+from sympy.core.add import Add
+from sympy.core.mul import Mul
+from sympy.core.singleton import S
 from sympy.polys.domains.characteristiczero import CharacteristicZero
 from sympy.polys.domains.field import Field
 from sympy.polys.domains.simpledomain import SimpleDomain
@@ -89,7 +92,7 @@ class AlgebraicField(Field, CharacteristicZero, SimpleDomain):
     to create instances which then support the operations ``+,-,*,**,/``. The
     :py:meth:`~.Domain.algebraic_field` method is used to construct a
     particular :ref:`QQ(a)` domain. The :py:meth:`~.Domain.from_sympy` method
-    can be used to create domain elements from normal sympy expressions.
+    can be used to create domain elements from normal SymPy expressions.
 
     >>> K = QQ.algebraic_field(sqrt(2))
     >>> K
@@ -178,6 +181,40 @@ class AlgebraicField(Field, CharacteristicZero, SimpleDomain):
     >>> K.mod
     DMP([1, 0, -10, 0, 1], QQ, None)
 
+    The `discriminant`_ of the field can be obtained from the
+    :py:meth:`~.discriminant` method, and an `integral basis`_ from the
+    :py:meth:`~.integral_basis` method. The latter returns a list of
+    :py:class:`~.ANP` instances by default, but can be made to return instances
+    of :py:class:`~.Expr` or :py:class:`~.AlgebraicNumber` by passing a ``fmt``
+    argument. The maximal order, or ring of integers, of the field can also be
+    obtained from the :py:meth:`~.maximal_order` method, as a
+    :py:class:`~sympy.polys.numberfields.modules.Submodule`.
+
+    >>> zeta5 = exp(2*I*pi/5)
+    >>> K = QQ.algebraic_field(zeta5)
+    >>> K
+    QQ<exp(2*I*pi/5)>
+    >>> K.discriminant()
+    125
+    >>> K = QQ.algebraic_field(sqrt(5))
+    >>> K
+    QQ<sqrt(5)>
+    >>> K.integral_basis(fmt='sympy')
+    [1, 1/2 + sqrt(5)/2]
+    >>> K.maximal_order()
+    Submodule[[2, 0], [1, 1]]/2
+
+    The factorization of a rational prime into prime ideals of the field is
+    computed by the :py:meth:`~.primes_above` method, which returns a list
+    of :py:class:`~sympy.polys.numberfields.primes.PrimeIdeal` instances.
+
+    >>> zeta7 = exp(2*I*pi/7)
+    >>> K = QQ.algebraic_field(zeta7)
+    >>> K
+    QQ<exp(2*I*pi/7)>
+    >>> K.primes_above(11)
+    [(11, _x**3 + 5*_x**2 + 4*_x - 1), (11, _x**3 - 4*_x**2 - 5*_x - 1)]
+
     Notes
     =====
 
@@ -192,6 +229,8 @@ class AlgebraicField(Field, CharacteristicZero, SimpleDomain):
 
     .. _algebraic number field: https://en.wikipedia.org/wiki/Algebraic_number_field
     .. _algebraic numbers: https://en.wikipedia.org/wiki/Algebraic_number
+    .. _discriminant: https://en.wikipedia.org/wiki/Discriminant_of_an_algebraic_number_field
+    .. _integral basis: https://en.wikipedia.org/wiki/Algebraic_number_field#Integral_basis
     .. _minimal polynomial: https://en.wikipedia.org/wiki/Minimal_polynomial_(field_theory)
     .. _primitive element: https://en.wikipedia.org/wiki/Primitive_element_theorem
     """
@@ -204,7 +243,25 @@ class AlgebraicField(Field, CharacteristicZero, SimpleDomain):
     has_assoc_Ring = False
     has_assoc_Field = True
 
-    def __init__(self, dom, *ext):
+    def __init__(self, dom, *ext, alias=None):
+        r"""
+        Parameters
+        ==========
+
+        dom : :py:class:`~.Domain`
+            The base field over which this is an extension field.
+            Currently only :ref:`QQ` is accepted.
+
+        *ext : One or more :py:class:`~.Expr`
+            Generators of the extension. These should be expressions that are
+            algebraic over `\mathbb{Q}`.
+
+        alias : str, :py:class:`~.Symbol`, None, optional (default=None)
+            If provided, this will be used as the alias symbol for the
+            primitive element of the :py:class:`~.AlgebraicField`.
+            If ``None``, while ``ext`` consists of exactly one
+            :py:class:`~.AlgebraicNumber`, its alias (if any) will be used.
+        """
         if not dom.is_QQ:
             raise DomainError("ground domain must be a rational field")
 
@@ -214,6 +271,9 @@ class AlgebraicField(Field, CharacteristicZero, SimpleDomain):
         else:
             orig_ext = ext
 
+        if alias is None and len(ext) == 1:
+            alias = getattr(ext[0], 'alias', None)
+
         self.orig_ext = orig_ext
         """
         Original elements given to generate the extension.
@@ -224,7 +284,7 @@ class AlgebraicField(Field, CharacteristicZero, SimpleDomain):
         (sqrt(2), sqrt(3))
         """
 
-        self.ext = to_number_field(ext)
+        self.ext = to_number_field(ext, alias=alias)
         """
         Primitive element used for the extension.
 
@@ -253,6 +313,10 @@ class AlgebraicField(Field, CharacteristicZero, SimpleDomain):
         self.zero = self.dtype.zero(self.mod.rep, dom)
         self.one = self.dtype.one(self.mod.rep, dom)
 
+        self._maximal_order = None
+        self._discriminant = None
+        self._nilradicals_mod_p = {}
+
     def new(self, element):
         return self.dtype(element, self.mod.rep, self.dom)
 
@@ -267,12 +331,16 @@ class AlgebraicField(Field, CharacteristicZero, SimpleDomain):
         return isinstance(other, AlgebraicField) and \
             self.dtype == other.dtype and self.ext == other.ext
 
-    def algebraic_field(self, *extension):
+    def algebraic_field(self, *extension, alias=None):
         r"""Returns an algebraic field, i.e. `\mathbb{Q}(\alpha, \ldots)`. """
-        return AlgebraicField(self.dom, *((self.ext,) + extension))
+        return AlgebraicField(self.dom, *((self.ext,) + extension), alias=alias)
+
+    def to_alg_num(self, a):
+        """Convert ``a`` of ``dtype`` to an :py:class:`~.AlgebraicNumber`. """
+        return self.ext.field_element(a)
 
     def to_sympy(self, a):
-        """Convert ``a`` to a SymPy object. """
+        """Convert ``a`` of ``dtype`` to a SymPy object. """
         # Precompute a converter to be reused:
         if not hasattr(self, '_converter'):
             self._converter = _make_converter(self)
@@ -362,15 +430,109 @@ class AlgebraicField(Field, CharacteristicZero, SimpleDomain):
         """Convert a GaussianRational element 'a' to ``dtype``. """
         return K1.from_sympy(K0.to_sympy(a))
 
+    def _do_round_two(self):
+        from sympy.polys.numberfields.basis import round_two
+        ZK, dK = round_two(self, radicals=self._nilradicals_mod_p)
+        self._maximal_order = ZK
+        self._discriminant = dK
+
+    def maximal_order(self):
+        """
+        Compute the maximal order, or ring of integers, of the field.
+
+        Returns
+        =======
+
+        :py:class:`~sympy.polys.numberfields.modules.Submodule`.
+
+        See Also
+        ========
+
+        integral_basis()
+
+        """
+        if self._maximal_order is None:
+            self._do_round_two()
+        return self._maximal_order
+
+    def integral_basis(self, fmt=None):
+        r"""
+        Get an integral basis for the field.
+
+        Parameters
+        ==========
+
+        fmt : str, None, optional (default=None)
+            If ``None``, return a list of :py:class:`~.ANP` instances.
+            If ``"sympy"``, convert each element of the list to an
+            :py:class:`~.Expr`, using ``self.to_sympy()``.
+            If ``"alg"``, convert each element of the list to an
+            :py:class:`~.AlgebraicNumber`, using ``self.to_alg_num()``.
+
+        Examples
+        ========
+
+        >>> from sympy import QQ, AlgebraicNumber, sqrt
+        >>> alpha = AlgebraicNumber(sqrt(5), alias='alpha')
+        >>> k = QQ.algebraic_field(alpha)
+        >>> B0 = k.integral_basis()
+        >>> B1 = k.integral_basis(fmt='sympy')
+        >>> B2 = k.integral_basis(fmt='alg')
+        >>> print(B0[1])  # doctest: +SKIP
+        ANP([mpq(1,2), mpq(1,2)], [mpq(1,1), mpq(0,1), mpq(-5,1)], QQ)
+        >>> print(B1[1])
+        1/2 + alpha/2
+        >>> print(B2[1])
+        alpha/2 + 1/2
+
+        In the last two cases we get legible expressions, which print somewhat
+        differently because of the different types involved:
+
+        >>> print(type(B1[1]))
+        <class 'sympy.core.add.Add'>
+        >>> print(type(B2[1]))
+        <class 'sympy.core.numbers.AlgebraicNumber'>
+
+        See Also
+        ========
+
+        to_sympy()
+        to_alg_num()
+        maximal_order()
+
+        """
+        ZK = self.maximal_order()
+        M = ZK.QQ_matrix
+        n = M.shape[1]
+        B = [self.new(list(reversed(M[:, j].flat()))) for j in range(n)]
+        if fmt == 'sympy':
+            return [self.to_sympy(b) for b in B]
+        elif fmt == 'alg':
+            return [self.to_alg_num(b) for b in B]
+        return B
+
+    def discriminant(self):
+        """Get the discriminant of the field."""
+        if self._discriminant is None:
+            self._do_round_two()
+        return self._discriminant
+
+    def primes_above(self, p):
+        """Compute the prime ideals lying above a given rational prime *p*."""
+        from sympy.polys.numberfields.primes import prime_decomp
+        ZK = self.maximal_order()
+        dK = self.discriminant()
+        rad = self._nilradicals_mod_p.get(p)
+        return prime_decomp(p, ZK=ZK, dK=dK, radical=rad)
+
 
 def _make_converter(K):
     """Construct the converter to convert back to Expr"""
-    # Precompute the effect of converting to sympy and expanding expressions
+    # Precompute the effect of converting to SymPy and expanding expressions
     # like (sqrt(2) + sqrt(3))**2. Asking Expr to do the expansion on every
     # conversion from K to Expr is slow. Here we compute the expansions for
     # each power of the generator and collect together the resulting algebraic
     # terms and the rational coefficients into a matrix.
-    from sympy import Add, S
 
     gen = K.ext.as_expr()
     todom = K.dom.from_sympy
@@ -384,7 +546,7 @@ def _make_converter(K):
         powers.append((gen * powers[-1]).expand())
 
     # Collect the rational coefficients and algebraic Expr that can
-    # map the ANP coefficients into an expanded sympy expression
+    # map the ANP coefficients into an expanded SymPy expression
     terms = [dict(t.as_coeff_Mul()[::-1] for t in Add.make_args(p)) for p in powers]
     algebraics = set().union(*terms)
     matrix = [[todom(t.get(a, S.Zero)) for t in terms] for a in algebraics]
@@ -393,7 +555,6 @@ def _make_converter(K):
 
     def converter(a):
         """Convert a to Expr using converter"""
-        from sympy import Add, Mul
         ai = a.rep[::-1]
         tosympy = K.dom.to_sympy
         coeffs_dom = [sum(mij*aj for mij, aj in zip(mi, ai)) for mi in matrix]
