@@ -1,8 +1,8 @@
-from typing import Any, Dict
+from typing import Any, Dict as tDict
 
-from sympy.core.compatibility import is_sequence
 from sympy.external import import_module
 from sympy.printing.printer import Printer
+from sympy.utilities.iterables import is_sequence
 import sympy
 from functools import partial
 
@@ -19,7 +19,7 @@ if aesara:
     mapping = {
             sympy.Add: aet.add,
             sympy.Mul: aet.mul,
-            sympy.Abs: aet.abs_,
+            sympy.Abs: aet.abs,
             sympy.sign: aet.sgn,
             sympy.ceiling: aet.ceil,
             sympy.floor: aet.floor,
@@ -51,8 +51,10 @@ if aesara:
             sympy.StrictLessThan: aet.lt,
             sympy.LessThan: aet.le,
             sympy.GreaterThan: aet.ge,
-            sympy.And: aet.and_,
-            sympy.Or: aet.or_,
+            sympy.And: aet.bitwise_and,  # bitwise
+            sympy.Or: aet.bitwise_or,  # bitwise
+            sympy.Not: aet.invert,  # bitwise
+            sympy.Xor: aet.bitwise_xor,  # bitwise
             sympy.Max: aet.maximum,  # Sympy accept >2 inputs, Aesara only 2
             sympy.Min: aet.minimum,  # Sympy accept >2 inputs, Aesara only 2
             sympy.conjugate: aet.conj,
@@ -86,7 +88,7 @@ class AesaraPrinter(Printer):
     ==========
 
     cache : dict
-        A cache of Aesara variables which have been created for Sympy
+        A cache of Aesara variables which have been created for SymPy
         symbol-like objects (e.g. :class:`sympy.core.symbol.Symbol` or
         :class:`sympy.matrices.expressions.MatrixSymbol`). This is used to
         ensure that all references to a given symbol in an expression (or
@@ -97,17 +99,17 @@ class AesaraPrinter(Printer):
     printmethod = "_aesara"
 
     def __init__(self, *args, **kwargs):
-        self.cache = kwargs.pop('cache', dict())
+        self.cache = kwargs.pop('cache', {})
         super().__init__(*args, **kwargs)
 
     def _get_key(self, s, name=None, dtype=None, broadcastable=None):
-        """ Get the cache key for a Sympy object.
+        """ Get the cache key for a SymPy object.
 
         Parameters
         ==========
 
         s : sympy.core.basic.Basic
-            Sympy object to get key for.
+            SymPy object to get key for.
 
         name : str
             Name of object, if it does not have a ``name`` attribute.
@@ -120,7 +122,7 @@ class AesaraPrinter(Printer):
 
     def _get_or_create(self, s, name=None, dtype=None, broadcastable=None):
         """
-        Get the Aesara variable for a Sympy symbol from the cache, or create it
+        Get the Aesara variable for a SymPy symbol from the cache, or create it
         if it does not exist.
         """
 
@@ -257,12 +259,12 @@ class AesaraPrinter(Printer):
         return expr
 
     def doprint(self, expr, dtypes=None, broadcastables=None):
-        """ Convert a Sympy expression to a Aesara graph variable.
+        """ Convert a SymPy expression to a Aesara graph variable.
 
         The ``dtypes`` and ``broadcastables`` arguments are used to specify the
         data type, dimension, and broadcasting behavior of the Aesara variables
         corresponding to the free symbols in ``expr``. Each is a mapping from
-        Sympy symbols to the value of the corresponding argument to
+        SymPy symbols to the value of the corresponding argument to
         ``aesara.tensor.var.TensorVariable``.
 
         See the corresponding `documentation page`__ for more information on
@@ -274,16 +276,16 @@ class AesaraPrinter(Printer):
         ==========
 
         expr : sympy.core.expr.Expr
-            Sympy expression to print.
+            SymPy expression to print.
 
         dtypes : dict
-            Mapping from Sympy symbols to Aesara datatypes to use when creating
+            Mapping from SymPy symbols to Aesara datatypes to use when creating
             new Aesara variables for those symbols. Corresponds to the ``dtype``
             argument to ``aesara.tensor.var.TensorVariable``. Defaults to ``'floatX'``
             for symbols not included in the mapping.
 
         broadcastables : dict
-            Mapping from Sympy symbols to the value of the ``broadcastable``
+            Mapping from SymPy symbols to the value of the ``broadcastable``
             argument to ``aesara.tensor.var.TensorVariable`` to use when creating Aesara
             variables for those symbols. Defaults to the empty tuple for symbols
             not included in the mapping (resulting in a scalar).
@@ -304,18 +306,18 @@ class AesaraPrinter(Printer):
         return self._print(expr, dtypes=dtypes, broadcastables=broadcastables)
 
 
-global_cache = {}  # type: Dict[Any, Any]
+global_cache = {}  # type: tDict[Any, Any]
 
 
 def aesara_code(expr, cache=None, **kwargs):
     """
-    Convert a Sympy expression into a Aesara graph variable.
+    Convert a SymPy expression into a Aesara graph variable.
 
     Parameters
     ==========
 
     expr : sympy.core.expr.Expr
-        Sympy expression object to convert.
+        SymPy expression object to convert.
 
     cache : dict
         Cached Aesara variables (see :class:`AesaraPrinter.cache
