@@ -6,22 +6,26 @@ from sympy.codegen.matrix_nodes import MatrixSolve
 from sympy.core import Expr, Mod, symbols, Eq, Le, Gt, zoo, oo, Rational, Pow
 from sympy.core.numbers import pi
 from sympy.core.singleton import S
-from sympy.functions import acos, KroneckerDelta, Piecewise, sign, sqrt
+from sympy.functions import acos, KroneckerDelta, Piecewise, sign, sqrt, Min, Max, cot, acsch, asec, coth
 from sympy.logic import And, Or
 from sympy.matrices import SparseMatrix, MatrixSymbol, Identity
 from sympy.printing.pycode import (
     MpmathPrinter, PythonCodePrinter, pycode, SymPyPrinter
 )
+from sympy.printing.tensorflow import TensorflowPrinter
 from sympy.printing.numpy import NumPyPrinter, SciPyPrinter
-from sympy.testing.pytest import raises
-from sympy.tensor import IndexedBase
-from sympy.testing.pytest import skip
+from sympy.testing.pytest import raises, skip
+from sympy.tensor import IndexedBase, Idx
+from sympy.tensor.array.expressions.array_expressions import ArraySymbol, ArrayDiagonal, ArrayContraction, ZeroArray, OneArray
 from sympy.external import import_module
 from sympy.functions.special.gamma_functions import loggamma
+from sympy.parsing.latex import parse_latex
+
 
 x, y, z = symbols('x y z')
 p = IndexedBase("p")
 
+
 def test_PythonCodePrinter():
     prntr = PythonCodePrinter()
 
@@ -29,8 +33,11 @@ def test_PythonCodePrinter():
 
     assert prntr.doprint(x**y) == 'x**y'
     assert prntr.doprint(Mod(x, 2)) == 'x % 2'
+    assert prntr.doprint(-Mod(x, y)) == '-(x % y)'
+    assert prntr.doprint(Mod(-x, y)) == '(-x) % y'
     assert prntr.doprint(And(x, y)) == 'x and y'
     assert prntr.doprint(Or(x, y)) == 'x or y'
+    assert prntr.doprint(1/(x+y)) == '1/(x + y)'
     assert not prntr.module_imports
 
     assert prntr.doprint(pi) == 'math.pi'
@@ -41,6 +48,11 @@ def test_PythonCodePrinter():
     assert prntr.module_imports == {'math': {'pi', 'sqrt'}}
 
     assert prntr.doprint(acos(x)) == 'math.acos(x)'
+    assert prntr.doprint(cot(x)) == '1/math.tan(x)'
+    assert prntr.doprint(coth(x)) == '(math.exp(x) + math.exp(-x))/(math.exp(x) - math.exp(-x))'
+    assert prntr.doprint(asec(x)) == 'math.acos(1/x)'
+    assert prntr.doprint(acsch(x)) == 'math.log(math.sqrt(1 + x**(-2)) + 1/x)'
+
     assert prntr.doprint(Assignment(x, 2)) == 'x = 2'
     assert prntr.doprint(Piecewise((1, Eq(x, 0)),
                         (2, x>6))) == '((1) if (x == 0) else (2) if (x > 6) else None)'
@@ -51,19 +63,21 @@ def test_PythonCodePrinter():
     assert prntr.doprint(p[0, 1]) == 'p[0, 1]'
     assert prntr.doprint(KroneckerDelta(x,y)) == '(1 if x == y else 0)'
 
+    assert prntr.doprint((2,3)) == "(2, 3)"
+    assert prntr.doprint([2,3]) == "[2, 3]"
+
+    assert prntr.doprint(Min(x, y)) == "min(x, y)"
+    assert prntr.doprint(Max(x, y)) == "max(x, y)"
+
 
 def test_PythonCodePrinter_standard():
-    import sys
-    prntr = PythonCodePrinter({'standard':None})
+    prntr = PythonCodePrinter()
 
-    python_version = sys.version_info.major
-    if python_version == 2:
-        assert prntr.standard == 'python2'
-    if python_version == 3:
-        assert prntr.standard == 'python3'
+    assert prntr.standard == 'python3'
 
     raises(ValueError, lambda: PythonCodePrinter({'standard':'python4'}))
 
+
 def test_MpmathPrinter():
     p = MpmathPrinter()
     assert p.doprint(sign(x)) == 'mpmath.sign(x)'
@@ -80,9 +94,13 @@ def test_MpmathPrinter():
 
 
 def test_NumPyPrinter():
-    from sympy import (Lambda, ZeroMatrix, OneMatrix, FunctionMatrix,
-        HadamardProduct, KroneckerProduct, Adjoint, DiagonalOf,
-        DiagMatrix, DiagonalMatrix)
+    from sympy.core.function import Lambda
+    from sympy.matrices.expressions.adjoint import Adjoint
+    from sympy.matrices.expressions.diagonal import (DiagMatrix, DiagonalMatrix, DiagonalOf)
+    from sympy.matrices.expressions.funcmatrix import FunctionMatrix
+    from sympy.matrices.expressions.hadamard import HadamardProduct
+    from sympy.matrices.expressions.kronecker import KroneckerProduct
+    from sympy.matrices.expressions.special import (OneMatrix, ZeroMatrix)
     from sympy.abc import a, b
     p = NumPyPrinter()
     assert p.doprint(sign(x)) == 'numpy.sign(x)'
@@ -130,16 +148,17 @@ def test_issue_18770():
     if not numpy:
         skip("numpy not installed.")
 
-    from sympy import lambdify, Min, Max
+    from sympy.functions.elementary.miscellaneous import (Max, Min)
+    from sympy.utilities.lambdify import lambdify
 
     expr1 = Min(0.1*x + 3, x + 1, 0.5*x + 1)
     func = lambdify(x, expr1, "numpy")
-    assert (func(numpy.linspace(0, 3, 3)) == [1.0 , 1.75, 2.5 ]).all()
+    assert (func(numpy.linspace(0, 3, 3)) == [1.0, 1.75, 2.5 ]).all()
     assert  func(4) == 3
 
-    expr1 = Max(x**2 , x**3)
+    expr1 = Max(x**2, x**3)
     func = lambdify(x,expr1, "numpy")
-    assert (func(numpy.linspace(-1 , 2, 4)) == [1, 0, 1, 8] ).all()
+    assert (func(numpy.linspace(-1, 2, 4)) == [1, 0, 1, 8] ).all()
     assert func(4) == 64
 
 
@@ -167,15 +186,22 @@ def test_pycode_reserved_words():
     assert py_str in ('else_ + if_', 'if_ + else_')
 
 
+def test_issue_20762():
+    antlr4 = import_module("antlr4")
+    if not antlr4:
+        skip('antlr not installed.')
+    # Make sure pycode removes curly braces from subscripted variables
+    expr = parse_latex(r'a_b \cdot b')
+    assert pycode(expr) == 'a_b*b'
+    expr = parse_latex(r'a_{11} \cdot b')
+    assert pycode(expr) == 'a_11*b'
+
+
 def test_sqrt():
     prntr = PythonCodePrinter()
     assert prntr._print_Pow(sqrt(x), rational=False) == 'math.sqrt(x)'
     assert prntr._print_Pow(1/sqrt(x), rational=False) == '1/math.sqrt(x)'
 
-    prntr = PythonCodePrinter({'standard' : 'python2'})
-    assert prntr._print_Pow(sqrt(x), rational=True) == 'x**(1./2.)'
-    assert prntr._print_Pow(1/sqrt(x), rational=True) == 'x**(-1./2.)'
-
     prntr = PythonCodePrinter({'standard' : 'python3'})
     assert prntr._print_Pow(sqrt(x), rational=True) == 'x**(1/2)'
     assert prntr._print_Pow(1/sqrt(x), rational=True) == 'x**(-1/2)'
@@ -199,10 +225,9 @@ def test_sqrt():
 
 
 def test_frac():
-    from sympy import frac
+    from sympy.functions.elementary.integers import frac
 
     expr = frac(x)
-
     prntr = NumPyPrinter()
     assert prntr.doprint(expr) == 'numpy.mod(x, 1)'
 
@@ -240,9 +265,10 @@ def test_codegen_ast_nodes():
 def test_issue_14283():
     prntr = PythonCodePrinter()
 
-    assert prntr.doprint(zoo) == "float('nan')"
+    assert prntr.doprint(zoo) == "math.nan"
     assert prntr.doprint(-oo) == "float('-inf')"
 
+
 def test_NumPyPrinter_print_seq():
     n = NumPyPrinter()
 
@@ -250,7 +276,7 @@ def test_NumPyPrinter_print_seq():
 
 
 def test_issue_16535_16536():
-    from sympy import lowergamma, uppergamma
+    from sympy.functions.special.gamma_functions import (lowergamma, uppergamma)
 
     a = symbols('a')
     expr1 = lowergamma(a, x)
@@ -270,7 +296,8 @@ def test_issue_16535_16536():
 
 
 def test_Integral():
-    from sympy import Integral, exp
+    from sympy.functions.elementary.exponential import exp
+    from sympy.integrals.integrals import Integral
 
     single = Integral(exp(-x), (x, 0, oo))
     double = Integral(x**2*exp(x*y), (x, -z, z), (y, 0, z))
@@ -291,7 +318,7 @@ def test_Integral():
 
 
 def test_fresnel_integrals():
-    from sympy import fresnelc, fresnels
+    from sympy.functions.special.error_functions import (fresnelc, fresnels)
 
     expr1 = fresnelc(x)
     expr2 = fresnels(x)
@@ -314,7 +341,7 @@ def test_fresnel_integrals():
 
 
 def test_beta():
-    from sympy import beta
+    from sympy.functions.special.beta_functions import beta
 
     expr = beta(x, y)
 
@@ -334,7 +361,7 @@ def test_beta():
     assert prntr.doprint(expr) ==  'mpmath.beta(x, y)'
 
 def test_airy():
-    from sympy import airyai, airybi
+    from sympy.functions.special.bessel import (airyai, airybi)
 
     expr1 = airyai(x)
     expr2 = airybi(x)
@@ -352,7 +379,7 @@ def test_airy():
     assert "Not supported" in prntr.doprint(expr2)
 
 def test_airy_prime():
-    from sympy import airyaiprime, airybiprime
+    from sympy.functions.special.bessel import (airyaiprime, airybiprime)
 
     expr1 = airyaiprime(x)
     expr2 = airybiprime(x)
@@ -375,3 +402,28 @@ def test_numerical_accuracy_functions():
     assert prntr.doprint(expm1(x)) == 'numpy.expm1(x)'
     assert prntr.doprint(log1p(x)) == 'numpy.log1p(x)'
     assert prntr.doprint(cosm1(x)) == 'scipy.special.cosm1(x)'
+
+def test_array_printer():
+    A = ArraySymbol('A', (4,4,6,6,6))
+    I = IndexedBase('I')
+    i,j,k = Idx('i', (0,1)), Idx('j', (2,3)), Idx('k', (4,5))
+
+    prntr = NumPyPrinter()
+    assert prntr.doprint(ZeroArray(5)) == 'numpy.zeros((5,))'
+    assert prntr.doprint(OneArray(5)) == 'numpy.ones((5,))'
+    assert prntr.doprint(ArrayContraction(A, [2,3])) == 'numpy.einsum("abccd->abd", A)'
+    assert prntr.doprint(I) == 'I'
+    assert prntr.doprint(ArrayDiagonal(A, [2,3,4])) == 'numpy.einsum("abccc->abc", A)'
+    assert prntr.doprint(ArrayDiagonal(A, [0,1], [2,3])) == 'numpy.einsum("aabbc->cab", A)'
+    assert prntr.doprint(ArrayContraction(A, [2], [3])) == 'numpy.einsum("abcde->abe", A)'
+    assert prntr.doprint(Assignment(I[i,j,k], I[i,j,k])) == 'I = I'
+
+    prntr = TensorflowPrinter()
+    assert prntr.doprint(ZeroArray(5)) == 'tensorflow.zeros((5,))'
+    assert prntr.doprint(OneArray(5)) == 'tensorflow.ones((5,))'
+    assert prntr.doprint(ArrayContraction(A, [2,3])) == 'tensorflow.linalg.einsum("abccd->abd", A)'
+    assert prntr.doprint(I) == 'I'
+    assert prntr.doprint(ArrayDiagonal(A, [2,3,4])) == 'tensorflow.linalg.einsum("abccc->abc", A)'
+    assert prntr.doprint(ArrayDiagonal(A, [0,1], [2,3])) == 'tensorflow.linalg.einsum("aabbc->cab", A)'
+    assert prntr.doprint(ArrayContraction(A, [2], [3])) == 'tensorflow.linalg.einsum("abcde->abe", A)'
+    assert prntr.doprint(Assignment(I[i,j,k], I[i,j,k])) == 'I = I'
