@@ -1,14 +1,38 @@
-from sympy import (Add, Abs, Catalan, cos, Derivative, E, EulerGamma, exp,
-    factorial, factorial2, Function, GoldenRatio, Heaviside,
-    TribonacciConstant, I, Integer, Integral, Interval, Lambda, Limit, Matrix,
-    nan, O, oo, pi, Pow, Rational, Float, Rel, S, sin, SparseMatrix, sqrt,
-    summation, Sum, Symbol, symbols, Wild, WildFunction, zeta, zoo, Dummy,
-    Dict, Tuple, FiniteSet, factor, subfactorial, true, false, Equivalent, Xor,
-    Complement, SymmetricDifference, AccumBounds, UnevaluatedExpr, Eq, Ne,
-    Quaternion, Subs, MatrixSymbol, MatrixSlice, Q,)
+from sympy import MatAdd
+from sympy.algebras.quaternion import Quaternion
+from sympy.assumptions.ask import Q
+from sympy.calculus.accumulationbounds import AccumBounds
 from sympy.combinatorics.partitions import Partition
-from sympy.core import Expr, Mul
+from sympy.concrete.summations import (Sum, summation)
+from sympy.core.add import Add
+from sympy.core.containers import (Dict, Tuple)
+from sympy.core.expr import UnevaluatedExpr, Expr
+from sympy.core.function import (Derivative, Function, Lambda, Subs, WildFunction)
+from sympy.core.mul import Mul
+from sympy.core import (Catalan, EulerGamma, GoldenRatio, TribonacciConstant)
+from sympy.core.numbers import (E, Float, I, Integer, Rational, nan, oo, pi, zoo)
 from sympy.core.parameters import _exp_is_pow
+from sympy.core.power import Pow
+from sympy.core.relational import (Eq, Rel, Ne)
+from sympy.core.singleton import S
+from sympy.core.symbol import (Dummy, Symbol, Wild, symbols)
+from sympy.functions.combinatorial.factorials import (factorial, factorial2, subfactorial)
+from sympy.functions.elementary.complexes import Abs
+from sympy.functions.elementary.exponential import exp
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.trigonometric import (cos, sin)
+from sympy.functions.special.delta_functions import Heaviside
+from sympy.functions.special.zeta_functions import zeta
+from sympy.integrals.integrals import Integral
+from sympy.logic.boolalg import (Equivalent, false, true, Xor)
+from sympy.matrices.dense import Matrix
+from sympy.matrices.expressions.matexpr import MatrixSymbol
+from sympy.matrices.expressions.slice import MatrixSlice
+from sympy.matrices import SparseMatrix
+from sympy.polys.polytools import factor
+from sympy.series.limits import Limit
+from sympy.series.order import O
+from sympy.sets.sets import (Complement, FiniteSet, Interval, SymmetricDifference)
 from sympy.external import import_module
 from sympy.physics.control.lti import TransferFunction, Series, Parallel, \
     Feedback, TransferFunctionMatrix, MIMOSeries, MIMOParallel, MIMOFeedback
@@ -19,10 +43,10 @@ from sympy.geometry import Point, Circle, Polygon, Ellipse, Triangle
 from sympy.tensor import NDimArray
 from sympy.tensor.array.expressions.array_expressions import ArraySymbol, ArrayElement
 
-from sympy.testing.pytest import raises
+from sympy.testing.pytest import raises, warns_deprecated_sympy
 
 from sympy.printing import sstr, sstrrepr, StrPrinter
-from sympy.core.trace import Tr
+from sympy.physics.quantum.trace import Tr
 
 x, y, z, w, t = symbols('x,y,z,w,t')
 d = Dummy('d')
@@ -356,6 +380,12 @@ def test_Permutation_Cycle():
         assert sstr(p) == s
 
 
+    with warns_deprecated_sympy():
+        old_print_cyclic = Permutation.print_cyclic
+        Permutation.print_cyclic = False
+        assert sstr(Permutation([1, 0, 2])) == 'Permutation([1, 0, 2])'
+        Permutation.print_cyclic = old_print_cyclic
+
 def test_Pi():
     assert str(pi) == "pi"
 
@@ -916,7 +946,6 @@ def test_UniversalSet():
 
 
 def test_PrettyPoly():
-    from sympy.polys.domains import QQ
     F = QQ.frac_field(x, y)
     R = QQ[x, y]
     assert sstr(F.convert(x/(x + y))) == sstr(x/(x + y))
@@ -953,8 +982,6 @@ def test_issue_6387():
 
 
 def test_MatMul_MatAdd():
-    from sympy import MatrixSymbol
-
     X, Y = MatrixSymbol("X", 2, 2), MatrixSymbol("Y", 2, 2)
     assert str(2*(X + Y)) == "2*X + 2*Y"
 
@@ -962,6 +989,8 @@ def test_MatMul_MatAdd():
     assert str(-I*X) == "-I*X"
     assert str((1 + I)*X) == '(1 + I)*X'
     assert str(-(1 + I)*X) == '(-1 - I)*X'
+    assert str(MatAdd(MatAdd(X, Y), MatAdd(X, Y))) == '(X + Y) + (X + Y)'
+
 
 def test_MatrixSlice():
     n = Symbol('n', integer=True)
@@ -1083,6 +1112,10 @@ def test_issue_21823():
     assert str(Partition({1, 2})) == 'Partition({1, 2})'
 
 
+def test_issue_22689():
+    assert str(Mul(Pow(x,-2, evaluate=False), Pow(3,-1,evaluate=False), evaluate=False)) == "1/(x**2*3)"
+
+
 def test_issue_21119_21460():
     ss = lambda x: str(S(x, evaluate=False))
     assert ss('4/2') == '4/2'
@@ -1128,5 +1161,8 @@ def test_AppliedPredicate():
     assert sstr(Q.even(x)) == 'Q.even(x)'
 
 def test_printing_str_array_expressions():
-    assert sstr(ArraySymbol("A", 2, 3, 4)) == "A"
+    assert sstr(ArraySymbol("A", (2, 3, 4))) == "A"
     assert sstr(ArrayElement("A", (2, 1/(1-x), 0))) == "A[2, 1/(1 - x), 0]"
+    M = MatrixSymbol("M", 3, 3)
+    N = MatrixSymbol("N", 3, 3)
+    assert sstr(ArrayElement(M*N, [x, 0])) == "(M*N)[x, 0]"
