@@ -2,18 +2,21 @@ from collections import defaultdict
 
 from sympy import SYMPY_DEBUG
 
-from sympy.core import expand_power_base, sympify, Add, S, Mul, Derivative, Pow, symbols, expand_mul
-from sympy.core.add import _unevaluated_Add
-from sympy.core.compatibility import iterable, ordered, default_sort_key
-from sympy.core.parameters import global_parameters
+from sympy.core import sympify, S, Mul, Derivative, Pow
+from sympy.core.add import _unevaluated_Add, Add
+from sympy.core.assumptions import assumptions
 from sympy.core.exprtools import Factors, gcd_terms
-from sympy.core.function import _mexpand
-from sympy.core.mul import _keep_coeff, _unevaluated_Mul
+from sympy.core.function import _mexpand, expand_mul, expand_power_base
+from sympy.core.mul import _keep_coeff, _unevaluated_Mul, _mulsort
 from sympy.core.numbers import Rational, zoo, nan
+from sympy.core.parameters import global_parameters
+from sympy.core.sorting import ordered, default_sort_key
+from sympy.core.symbol import Dummy, Wild, symbols
 from sympy.functions import exp, sqrt, log
 from sympy.functions.elementary.complexes import Abs
 from sympy.polys import gcd
 from sympy.simplify.sqrtdenest import sqrtdenest
+from sympy.utilities.iterables import iterable, sift
 
 
 
@@ -111,13 +114,21 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
         (a + b)*exp(2*x)
 
     If you are interested only in collecting specific powers of some symbols
-    then set ``exact`` flag in arguments::
+    then set ``exact`` flag to True::
 
         >>> collect(a*x**7 + b*x**7, x, exact=True)
         a*x**7 + b*x**7
         >>> collect(a*x**7 + b*x**7, x**7, exact=True)
         x**7*(a + b)
 
+    If you want to collect on any object containing symbols, set
+    ``exact`` to None:
+
+        >>> collect(x*exp(x) + sin(x)*y + sin(x)*2 + 3*x, x, exact=None)
+        x*exp(x) + 3*x + (y + 2)*sin(x)
+        >>> collect(a*x*y + x*y + b*x + x, [x, y], exact=None)
+        x*y*(a + 1) + x*(b + 1)
+
     You can also apply this function to differential equations, where
     derivatives of arbitrary order can be collected. Note that if you
     collect with respect to a function or a derivative of a function, all
@@ -160,11 +171,9 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
 
     collect_const, collect_sqrt, rcollect
     """
-    from sympy.core.assumptions import assumptions
-    from sympy.utilities.iterables import sift
-    from sympy.core.symbol import Dummy, Wild
     expr = sympify(expr)
     syms = [sympify(i) for i in (syms if iterable(syms) else [syms])]
+
     # replace syms[i] if it is not x, -x or has Wild symbols
     cond = lambda x: x.is_Symbol or (-x).is_Symbol or bool(
         x.atoms(Wild))
@@ -182,6 +191,26 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
             return {urep.get(k, k).xreplace(urep): v.xreplace(urep)
                     for k, v in rv.items()}
 
+    # see if other expressions should be considered
+    if exact is None:
+        _syms = set()
+        for i in Add.make_args(expr):
+            if not i.has_free(*syms) or i in syms:
+                continue
+            if not i.is_Mul and i not in syms:
+                _syms.add(i)
+            else:
+                # identify compound generators
+                g = i._new_rawargs(*i.as_coeff_mul(*syms)[1])
+                if g not in syms:
+                    _syms.add(g)
+        simple = all(i.is_Pow and i.base in syms for i in _syms)
+        syms = syms + list(ordered(_syms))
+        if not simple:
+            return collect(expr, syms,
+            func=func, evaluate=evaluate, exact=False,
+            distribute_order_term=distribute_order_term)
+
     if evaluate is None:
         evaluate = global_parameters.evaluate
 
@@ -239,7 +268,7 @@ def collect(expr, syms, func=None, evaluate=None, exact=False, distribute_order_
          - sexpr is the base expression
          - rat_expo is the rational exponent that sexpr is raised to
          - sym_expo is the symbolic exponent that sexpr is raised to
-         - deriv contains the derivatives the the expression
+         - deriv contains the derivatives of the expression
 
          For example, the output of x would be (x, 1, None, None)
          the output of 2**x would be (2, 1, x, None).
@@ -576,7 +605,6 @@ def collect_abs(expr):
     Abs(1/x)
     """
     def _abs(mul):
-      from sympy.core.mul import _mulsort
       c, nc = mul.args_cnc()
       a = []
       o = []
@@ -620,7 +648,7 @@ def collect_const(expr, *vars, Numbers=True):
     Parameters
     ==========
 
-    expr : sympy expression
+    expr : SymPy expression
         This parameter defines the expression the expression from which
         terms with similar coefficients are to be collected. A non-Add
         expression is returned as it is.
@@ -952,7 +980,7 @@ def radsimp(expr, symbolic=True, max_terms=4):
                 # in general, only 4 terms can be removed with repeated squaring
                 # but other considerations can guide selection of radical terms
                 # so that radicals are removed
-                if all([x.is_Integer and (y**2).is_Rational for x, y in rterms]):
+                if all(x.is_Integer and (y**2).is_Rational for x, y in rterms):
                     nd, d = rad_rationalize(S.One, Add._from_args(
                         [sqrt(x)*y for x, y in rterms]))
                     n *= nd
