@@ -1,6 +1,17 @@
-from random import randint
-from sympy import (S, symbols, Function, Rational, Poly, Eq, ratsimp,
-    checkodesol, sqrt, Dummy, oo, I, Mul, sin, exp, log, tanh)
+from sympy.core.random import randint
+from sympy.core.function import Function
+from sympy.core.mul import Mul
+from sympy.core.numbers import (I, Rational, oo)
+from sympy.core.relational import Eq
+from sympy.core.singleton import S
+from sympy.core.symbol import (Dummy, symbols)
+from sympy.functions.elementary.exponential import (exp, log)
+from sympy.functions.elementary.hyperbolic import tanh
+from sympy.functions.elementary.miscellaneous import sqrt
+from sympy.functions.elementary.trigonometric import sin
+from sympy.polys.polytools import Poly
+from sympy.simplify.ratsimp import ratsimp
+from sympy.solvers.ode.subscheck import checkodesol
 from sympy.testing.pytest import slow
 from sympy.solvers.ode.riccati import (riccati_normal, riccati_inverse_normal,
     riccati_reduced, match_riccati, inverse_transform_poly, limit_at_inf,
