@@ -46,29 +46,27 @@ For the sake of completeness, `f(n)` can be:
     [2] a rational function        -> rsolve_ratio
     [3] a hypergeometric function  -> rsolve_hyper
 """
-from __future__ import print_function, division
-
 from collections import defaultdict
 
+from sympy.concrete import product
 from sympy.core.singleton import S
 from sympy.core.numbers import Rational, I
 from sympy.core.symbol import Symbol, Wild, Dummy
 from sympy.core.relational import Equality
 from sympy.core.add import Add
 from sympy.core.mul import Mul
-from sympy.core import sympify
+from sympy.core.sorting import default_sort_key
+from sympy.core.sympify import sympify
 
-from sympy.simplify import simplify, hypersimp, hypersimilar
+from sympy.simplify import simplify, hypersimp, hypersimilar  # type: ignore
 from sympy.solvers import solve, solve_undetermined_coeffs
 from sympy.polys import Poly, quo, gcd, lcm, roots, resultant
 from sympy.functions import binomial, factorial, FallingFactorial, RisingFactorial
 from sympy.matrices import Matrix, casoratian
-from sympy.concrete import product
-from sympy.core.compatibility import default_sort_key, range
 from sympy.utilities.iterables import numbered_symbols
 
 
-def rsolve_poly(coeffs, f, n, **hints):
+def rsolve_poly(coeffs, f, n, shift=0, **hints):
     r"""
     Given linear recurrence operator `\operatorname{L}` of order
     `k` with polynomial coefficients and inhomogeneous equation
@@ -136,7 +134,7 @@ def rsolve_poly(coeffs, f, n, **hints):
 
     for i in range(r + 1):
         for j in range(i, r + 1):
-            polys[i] += coeffs[j]*binomial(j, i)
+            polys[i] += coeffs[j]*(binomial(j, i).as_poly(n))
 
         if not polys[i].is_zero:
             (exp,), coeff = polys[i].LT()
@@ -190,7 +188,7 @@ def rsolve_poly(coeffs, f, n, **hints):
         y = E = S.Zero
 
         for i in range(N + 1):
-            C.append(Symbol('C' + str(i)))
+            C.append(Symbol('C' + str(i + shift)))
             y += C[i] * n**i
 
         for i in range(r + 1):
@@ -303,7 +301,7 @@ def rsolve_poly(coeffs, f, n, **hints):
         if not homogeneous:
             h = Add(*[(g*p).expand() for g, p in zip(G, P)])
 
-        C = [Symbol('C' + str(i)) for i in range(A)]
+        C = [Symbol('C' + str(i + shift)) for i in range(A)]
 
         g = lambda i: Add(*[c*_delta(q, i) for c, q in zip(C, Q)])
 
@@ -533,10 +531,7 @@ def rsolve_hyper(coeffs, f, n, **hints):
                 else:
                     similar[g] = S.Zero
 
-            inhomogeneous = []
-
-            for g, h in similar.items():
-                inhomogeneous.append(g + h)
+            inhomogeneous = [g + h for g, h in similar.items()]
         elif f.is_hypergeometric(n):
             inhomogeneous = [f]
         else:
@@ -559,14 +554,24 @@ def rsolve_hyper(coeffs, f, n, **hints):
             for j in range(r + 1):
                 polys[j] *= Mul(*(denoms[:j] + denoms[j + 1:]))
 
-            R = rsolve_poly(polys, Mul(*denoms), n)
+            # FIXME: The call to rsolve_ratio below should suffice (rsolve_poly
+            # call can be removed) but the XFAIL test_rsolve_ratio_missed must
+            # be fixed first.
+            R = rsolve_ratio(polys, Mul(*denoms), n, symbols=True)
+            if R is not None:
+                R, syms = R
+                if syms:
+                    R = R.subs(zip(syms, [0]*len(syms)))
+            else:
+                R = rsolve_poly(polys, Mul(*denoms), n)
 
-            if not (R is None or R is S.Zero):
+            if R:
                 inhomogeneous[i] *= R
             else:
                 return None
 
             result = Add(*inhomogeneous)
+            result = simplify(result)
     else:
         result = S.Zero
 
@@ -620,11 +625,17 @@ def rsolve_hyper(coeffs, f, n, **hints):
             if z.is_zero:
                 continue
 
-            (C, s) = rsolve_poly([polys[i]*z**i for i in range(r + 1)], 0, n, symbols=True)
-
-            if C is not None and C is not S.Zero:
-                symbols |= set(s)
+            recurr_coeffs = [polys[i].as_expr()*z**i for i in range(r + 1)]
+            if d == 0 and 0 != Add(*[recurr_coeffs[j]*j for j in range(1, r + 1)]):
+                # faster inline check (than calling rsolve_poly) for a
+                # constant solution to a constant coefficient recurrence.
+                sol = [Symbol("C" + str(len(symbols)))]
+            else:
+                sol, syms = rsolve_poly(recurr_coeffs, 0, n, len(symbols), symbols=True)
+                sol = sol.collect(syms)
+                sol = [sol.coeff(s) for s in syms]
 
+            for C in sol:
                 ratio = z * A * C.subs(n, n + 1) / B / C
                 ratio = simplify(ratio)
                 # If there is a nonnegative root in the denominator of the ratio,
@@ -646,13 +657,11 @@ def rsolve_hyper(coeffs, f, n, **hints):
     kernel.sort(key=default_sort_key)
     sk = list(zip(numbered_symbols('C'), kernel))
 
-    if sk:
-        for C, ker in sk:
-            result += C * ker
-    else:
-        return None
+    for C, ker in sk:
+        result += C * ker
 
     if hints.get('symbols', False):
+        # XXX: This returns the symbols in a non-deterministic order
         symbols |= {s for s, k in sk}
         return (result, list(symbols))
     else:
@@ -721,37 +730,35 @@ def rsolve(f, y, init=None):
     # y(n) + a*(y(n + 1) + y(n - 1))/2
     f = f.expand().collect(y.func(Wild('m', integer=True)))
 
-    h_part = defaultdict(lambda: S.Zero)
-    i_part = S.Zero
+    h_part = defaultdict(list)
+    i_part = []
     for g in Add.make_args(f):
-        coeff = S.One
-        kspec = None
-        for h in Mul.make_args(g):
-            if h.is_Function:
-                if h.func == y.func:
-                    result = h.args[0].match(n + k)
-
-                    if result is not None:
-                        kspec = int(result[k])
-                    else:
-                        raise ValueError(
-                            "'%s(%s + k)' expected, got '%s'" % (y.func, n, h))
-                else:
-                    raise ValueError(
-                        "'%s' expected, got '%s'" % (y.func, h.func))
-            else:
-                coeff *= h
-
-        if kspec is not None:
-            h_part[kspec] += coeff
-        else:
-            i_part += coeff
+        coeff, dep = g.as_coeff_mul(y.func)
+        if not dep:
+            i_part.append(coeff)
+            continue
+        for h in dep:
+            if h.is_Function and h.func == y.func:
+                result = h.args[0].match(n + k)
+                if result is not None:
+                    h_part[int(result[k])].append(coeff)
+                    continue
+            raise ValueError(
+                "'%s(%s + k)' expected, got '%s'" % (y.func, n, h))
+    for k in h_part:
+        h_part[k] = Add(*h_part[k])
+    h_part.default_factory = lambda: 0
+    i_part = Add(*i_part)
 
     for k, coeff in h_part.items():
         h_part[k] = simplify(coeff)
 
     common = S.One
 
+    if not i_part.is_zero and not i_part.is_hypergeometric(n) and \
+       not (i_part.is_Add and all(map(lambda x: x.is_hypergeometric(n), i_part.expand().args))):
+        raise ValueError("The independent term should be a sum of hypergeometric functions, got '%s'" % i_part)
+
     for coeff in h_part.values():
         if coeff.is_rational_function(n):
             if not coeff.is_polynomial(n):
@@ -796,7 +803,7 @@ def rsolve(f, y, init=None):
 
     solution, symbols = result
 
-    if init == {} or init == []:
+    if init in ({}, []):
         init = None
 
     if symbols and init is not None:
@@ -813,10 +820,10 @@ def rsolve(f, y, init=None):
                     i = int(k.args[0])
                 else:
                     raise ValueError("Integer or term expected, got '%s'" % k)
-            try:
+
+            eq = solution.subs(n, i) - v
+            if eq.has(S.NaN):
                 eq = solution.limit(n, i) - v
-            except NotImplementedError:
-                eq = solution.subs(n, i) - v
             equations.append(eq)
 
         result = solve(equations, *symbols)
