@@ -1,10 +1,11 @@
-from sympy import Basic
-from sympy import S
+from sympy.core.basic import Basic
+from sympy.core.containers import (Dict, Tuple)
 from sympy.core.expr import Expr
-from sympy.core.numbers import Integer
-from sympy.core.sympify import sympify
 from sympy.core.kind import Kind, NumberKind, UndefinedKind
-from sympy.core.compatibility import SYMPY_INTS
+from sympy.core.numbers import Integer
+from sympy.core.singleton import S
+from sympy.core.sympify import sympify
+from sympy.external.gmpy import SYMPY_INTS
 from sympy.printing.defaults import Printable
 
 import itertools
@@ -51,7 +52,7 @@ class ArrayKind(Kind):
     the element kind. Use ``is`` with specifying the element kind.
 
     >>> from sympy.tensor.array import ArrayKind
-    >>> from sympy.core.kind import NumberKind
+    >>> from sympy.core import NumberKind
     >>> boolA = NDimArray([True, False])
     >>> isinstance(boolA.kind, ArrayKind)
     True
@@ -72,6 +73,15 @@ class ArrayKind(Kind):
     def __repr__(self):
         return "ArrayKind(%s)" % self.element_kind
 
+    @classmethod
+    def _union(cls, kinds) -> 'ArrayKind':
+        elem_kinds = set(e.kind for e in kinds)
+        if len(elem_kinds) == 1:
+            elemkind, = elem_kinds
+        else:
+            elemkind = UndefinedKind
+        return ArrayKind(elemkind)
+
 
 class NDimArray(Printable):
     """
@@ -133,21 +143,17 @@ class NDimArray(Printable):
         from sympy.tensor.array import ImmutableDenseNDimArray
         return ImmutableDenseNDimArray(iterable, shape, **kwargs)
 
-    @property
-    def kind(self):
-        elem_kinds = set(e.kind for e in self._array)
-        if len(elem_kinds) == 1:
-            elemkind, = elem_kinds
-        else:
-            elemkind = UndefinedKind
-        return ArrayKind(elemkind)
+    def __getitem__(self, index):
+        raise NotImplementedError("A subclass of NDimArray should implement __getitem__")
 
     def _parse_index(self, index):
         if isinstance(index, (SYMPY_INTS, Integer)):
-            raise ValueError("Only a tuple index is accepted")
+            if index >= self._loop_size:
+                raise ValueError("Only a tuple index is accepted")
+            return index
 
         if self._loop_size == 0:
-            raise ValueError("Index not valide with an empty array")
+            raise ValueError("Index not valid with an empty array")
 
         if len(index) != self._rank:
             raise ValueError('Wrong number of array axes')
@@ -174,7 +180,7 @@ class NDimArray(Printable):
     def _check_symbolic_index(self, index):
         # Check if any index is symbolic:
         tuple_index = (index if isinstance(index, tuple) else (index,))
-        if any([(isinstance(i, Expr) and (not i.is_number)) for i in tuple_index]):
+        if any((isinstance(i, Expr) and (not i.is_number)) for i in tuple_index):
             for i, nth_dim in zip(tuple_index, self.shape):
                 if ((i < 0) == True) or ((i >= nth_dim) == True):
                     raise ValueError("index out of range")
@@ -193,6 +199,9 @@ class NDimArray(Printable):
             if not isinstance(pointer, Iterable):
                 return [pointer], ()
 
+            if len(pointer) == 0:
+                return [], (0,)
+
             result = []
             elems, shapes = zip(*[f(i) for i in pointer])
             if len(set(shapes)) != 1:
@@ -207,7 +216,6 @@ class NDimArray(Printable):
     def _handle_ndarray_creation_inputs(cls, iterable=None, shape=None, **kwargs):
         from sympy.matrices.matrices import MatrixBase
         from sympy.tensor.array import SparseNDimArray
-        from sympy import Dict, Tuple
 
         if shape is None:
             if iterable is None:
@@ -246,7 +254,7 @@ class NDimArray(Printable):
         if isinstance(shape, (SYMPY_INTS, Integer)):
             shape = (shape,)
 
-        if any([not isinstance(dim, (SYMPY_INTS, Integer)) for dim in shape]):
+        if not all(isinstance(dim, (SYMPY_INTS, Integer)) for dim in shape):
             raise TypeError("Shape should contain integers only.")
 
         return tuple(shape), iterable
@@ -567,11 +575,11 @@ class NDimArray(Printable):
 
     def _check_index_for_getitem(self, index):
         if isinstance(index, (SYMPY_INTS, Integer, slice)):
-            index = (index, )
+            index = (index,)
 
         if len(index) < self.rank():
-            index = tuple([i for i in index] + \
-                          [slice(None) for i in range(len(index), self.rank())])
+            index = tuple(index) + \
+                          tuple(slice(None) for i in range(len(index), self.rank()))
 
         if len(index) > self.rank():
             raise ValueError('Dimension of index greater than rank of array')
