@@ -5,8 +5,7 @@ import subprocess
 import sys
 import tempfile
 import warnings
-from distutils.errors import CompileError
-from distutils.sysconfig import get_config_var, get_config_vars
+from sysconfig import get_config_var, get_config_vars, get_path
 
 from .runners import (
     CCompilerRunner,
@@ -16,11 +15,9 @@ from .runners import (
 from .util import (
     get_abspath, make_dirs, copy, Glob, ArbitraryDepthGlob,
     glob_at_depth, import_module_from_file, pyx_is_cplus,
-    sha256_of_string, sha256_of_file
+    sha256_of_string, sha256_of_file, CompileError
 )
 
-sharedext = get_config_var('EXT_SUFFIX')
-
 if os.name == 'posix':
     objext = '.o'
 elif os.name == 'nt':
@@ -149,7 +146,7 @@ def link(obj_files, out_file=None, shared=False, Runner=None,
     if out_file is None:
         out_file, ext = os.path.splitext(os.path.basename(obj_files[-1]))
         if shared:
-            out_file += sharedext
+            out_file += get_config_var('EXT_SUFFIX')
 
     if not Runner:
         if fort:
@@ -186,7 +183,7 @@ def link(obj_files, out_file=None, shared=False, Runner=None,
 
 def link_py_so(obj_files, so_file=None, cwd=None, libraries=None,
                cplus=False, fort=False, **kwargs):
-    """ Link python extension module (shared object) for importing
+    """ Link Python extension module (shared object) for importing
 
     Parameters
     ==========
@@ -228,8 +225,7 @@ def link_py_so(obj_files, so_file=None, cwd=None, libraries=None,
         # Don't use the default code below
         pass
     else:
-        from distutils import sysconfig
-        if sysconfig.get_config_var('Py_ENABLE_SHARED'):
+        if get_config_var('Py_ENABLE_SHARED'):
             cfgDict = get_config_vars()
             kwargs['linkline'] = kwargs.get('linkline', []) + [cfgDict['PY_LDFLAGS']] # PY_LDFLAGS or just LDFLAGS?
             library_dirs += [cfgDict['LIBDIR']]
@@ -288,6 +284,10 @@ def simple_cythonize(src, destdir=None, cwd=None, **cy_kwargs):
     try:
         cy_options = CompilationOptions(default_options)
         cy_options.__dict__.update(cy_kwargs)
+        # Set language_level if not set by cy_kwargs
+        # as not setting it is deprecated
+        if 'language_level' not in cy_kwargs:
+            cy_options.__dict__['language_level'] = 3
         cy_result = cy_compile([src], cy_options)
         if cy_result.num_errors > 0:
             raise ValueError("Cython compilation failed.")
@@ -350,8 +350,7 @@ def src2obj(srcpath, Runner=None, objpath=None, cwd=None, inc_py=False, **kwargs
 
     include_dirs = kwargs.pop('include_dirs', [])
     if inc_py:
-        from distutils.sysconfig import get_python_inc
-        py_inc_dir = get_python_inc()
+        py_inc_dir = get_path('include')
         if py_inc_dir not in include_dirs:
             include_dirs.append(py_inc_dir)
 
@@ -482,7 +481,7 @@ def any_cplus_src(srcs):
 
 def compile_link_import_py_ext(sources, extname=None, build_dir='.', compile_kwargs=None,
                                link_kwargs=None):
-    """ Compiles sources to a shared object (python extension) and imports it
+    """ Compiles sources to a shared object (Python extension) and imports it
 
     Sources in ``sources`` which is imported. If shared object is newer than the sources, they
     are not recompiled but instead it is imported.
@@ -505,7 +504,7 @@ def compile_link_import_py_ext(sources, extname=None, build_dir='.', compile_kwa
     Returns
     =======
 
-    The imported module from of the python extension.
+    The imported module from of the Python extension.
     """
     if extname is None:
         extname = os.path.splitext(os.path.basename(sources[-1]))[0]
@@ -536,7 +535,8 @@ def _write_sources_to_build_dir(sources, build_dir):
         sha256_in_mem = sha256_of_string(src.encode('utf-8')).hexdigest()
         if os.path.exists(dest):
             if os.path.exists(dest + '.sha256'):
-                sha256_on_disk = open(dest + '.sha256').read()
+                with open(dest + '.sha256') as fh:
+                    sha256_on_disk = fh.read()
             else:
                 sha256_on_disk = sha256_of_file(dest).hexdigest()
 
@@ -544,7 +544,8 @@ def _write_sources_to_build_dir(sources, build_dir):
         if differs:
             with open(dest, 'wt') as fh:
                 fh.write(src)
-                open(dest + '.sha256', 'wt').write(sha256_in_mem)
+            with open(dest + '.sha256', 'wt') as fh:
+                fh.write(sha256_in_mem)
         source_files.append(dest)
     return source_files, build_dir
 
