@@ -1,14 +1,13 @@
-"""Miscellaneous stuff that doesn't really fit anywhere else."""
+"""Miscellaneous stuff that does not really fit anywhere else."""
 
 from typing import List
 
+import operator
 import sys
 import os
 import re as _re
 import struct
 from textwrap import fill, dedent
-from sympy.core.compatibility import as_int
-from sympy.core.decorators import deprecated
 
 
 class Undecidable(ValueError):
@@ -17,20 +16,23 @@ class Undecidable(ValueError):
     pass
 
 
-def filldedent(s, w=70):
+def filldedent(s, w=70, **kwargs):
     """
-    Strips leading and trailing empty lines from a copy of `s`, then dedents,
+    Strips leading and trailing empty lines from a copy of ``s``, then dedents,
     fills and returns it.
 
     Empty line stripping serves to deal with docstrings like this one that
     start with a newline after the initial triple quote, inserting an empty
     line at the beginning of the string.
 
+    Additional keyword arguments will be passed to ``textwrap.fill()``.
+
     See Also
     ========
     strlines, rawlines
+
     """
-    return '\n' + fill(dedent(str(s)).strip('\n'), width=w)
+    return '\n' + fill(dedent(str(s)).strip('\n'), width=w, **kwargs)
 
 
 def strlines(s, c=64, short=False):
@@ -62,7 +64,7 @@ def strlines(s, c=64, short=False):
     ========
     filldedent, rawlines
     """
-    if type(s) is not str:
+    if not isinstance(s, str):
         raise ValueError('expecting string input')
     if '\n' in s:
         return rawlines(s)
@@ -170,7 +172,7 @@ def rawlines(s):
 ARCH = str(struct.calcsize('P') * 8) + "-bit"
 
 
-# XXX: PyPy doesn't support hash randomization
+# XXX: PyPy does not support hash randomization
 HASH_RANDOMIZATION = getattr(sys.flags, 'hash_randomization', False)
 
 _debug_tmp = []  # type: List[str]
@@ -193,13 +195,13 @@ def debug_decorator(func):
         _debug_iter += 1
 
         def tree(subtrees):
-            def indent(s, type=1):
+            def indent(s, variant=1):
                 x = s.split("\n")
                 r = "+-%s\n" % x[0]
                 for a in x[1:]:
                     if a == "":
                         continue
-                    if type == 1:
+                    if variant == 1:
                         r += "| %s\n" % a
                     else:
                         r += "  %s\n" % a
@@ -247,16 +249,21 @@ def debug(*args):
         print(*args, file=sys.stderr)
 
 
-@deprecated(
-    useinstead="the builtin ``shutil.which`` function",
-    issue=19634,
-    deprecated_since_version="1.7")
 def find_executable(executable, path=None):
     """Try to find 'executable' in the directories listed in 'path' (a
     string listing directories separated by 'os.pathsep'; defaults to
     os.environ['PATH']).  Returns the complete filename or None if not
     found
     """
+    from .exceptions import sympy_deprecation_warning
+    sympy_deprecation_warning(
+        """
+        sympy.utilities.misc.find_executable() is deprecated. Use the standard
+        library shutil.which() function instead.
+        """,
+        deprecated_since_version="1.7",
+        active_deprecations_target="deprecated-find-executable",
+    )
     if path is None:
         path = os.environ['PATH']
     paths = path.split(os.pathsep)
@@ -380,7 +387,7 @@ def replace(string, *reps):
     """
     if len(reps) == 1:
         kv = reps[0]
-        if type(kv) is dict:
+        if isinstance(kv, dict):
             reps = kv
         else:
             return string.replace(*kv)
@@ -440,7 +447,7 @@ def translate(s, a, b=None, c=None):
         c = b
         a = b = ''
     else:
-        if type(a) is dict:
+        if isinstance(a, dict):
             short = {}
             for k in list(a.keys()):
                 if len(k) == 1 and len(a[k]) == 1:
@@ -479,3 +486,70 @@ def ordinal(num):
     else:
         suffix = 'th'
     return str(n) + suffix
+
+
+def as_int(n, strict=True):
+    """
+    Convert the argument to a builtin integer.
+
+    The return value is guaranteed to be equal to the input. ValueError is
+    raised if the input has a non-integral value. When ``strict`` is True, this
+    uses `__index__ <https://docs.python.org/3/reference/datamodel.html#object.__index__>`_
+    and when it is False it uses ``int``.
+
+
+    Examples
+    ========
+
+    >>> from sympy.utilities.misc import as_int
+    >>> from sympy import sqrt, S
+
+    The function is primarily concerned with sanitizing input for
+    functions that need to work with builtin integers, so anything that
+    is unambiguously an integer should be returned as an int:
+
+    >>> as_int(S(3))
+    3
+
+    Floats, being of limited precision, are not assumed to be exact and
+    will raise an error unless the ``strict`` flag is False. This
+    precision issue becomes apparent for large floating point numbers:
+
+    >>> big = 1e23
+    >>> type(big) is float
+    True
+    >>> big == int(big)
+    True
+    >>> as_int(big)
+    Traceback (most recent call last):
+    ...
+    ValueError: ... is not an integer
+    >>> as_int(big, strict=False)
+    99999999999999991611392
+
+    Input that might be a complex representation of an integer value is
+    also rejected by default:
+
+    >>> one = sqrt(3 + 2*sqrt(2)) - sqrt(2)
+    >>> int(one) == 1
+    True
+    >>> as_int(one)
+    Traceback (most recent call last):
+    ...
+    ValueError: ... is not an integer
+    """
+    if strict:
+        try:
+            if isinstance(n, bool):
+                raise TypeError
+            return operator.index(n)
+        except TypeError:
+            raise ValueError('%s is not an integer' % (n,))
+    else:
+        try:
+            result = int(n)
+        except TypeError:
+            raise ValueError('%s is not an integer' % (n,))
+        if n != result:
+            raise ValueError('%s is not an integer' % (n,))
+        return result
